#<profile4xi profile = "03" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "popen4xi.py">
#<abstract4xi abstract = "provides class Popen4Xi"/>
#<refer4xi reference = "forking.py"/>
#<state4xi status = "ACK4tagging; NACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Popen4Xi']
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None
                
try:                 # most-common ones
    import itertools
    import queue
    import time            
except:
    pass
try:                 # rare ones
    from pickle import load, HIGHEST_PROTOCOL 
    import collections
    import msvcrt                
except:
    pass 
try:                 # imported-as ones 
    from pickle import _Pickler as Pickler          
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing import forking
    from multiprocessing.forking import Popen
    from multiprocessing.forking import ForkingPickler
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from close4xi import close4xi
from constant_forking4xi import ConstantForking4Xi #dragonized import for TERMINATE4XI and WINEXE4XI 
from constant_system4xi import get_windows_system4xi
from current_process4xi import set_inheriting_current_process4xi
from current_process4xi import get_authkey_current_process4xi
from current_process4xi import get_identity_current_process4xi
from duplicate4xi import duplicate4xi
from examine4xi import examine4xi
from finalize4xi import Finalize4Xi
from forkingpickler4xi import ForkingPickler4Xi
from get_preparation_data4xi import get_preparation_data4xi
from loop4xi import limit2loop4xi
from original_dir4xi import get_original_dir4xi
from prepare4xi import prepare4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from sleep4xi import omega2sleep4xi
from _reduce_method4xi import _reduce_method4xi
from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi        
#"
#/>




# -------------------------------------------------------------------------------------------------    
#
# -------------------------------------------------------------------------------------------------
if sys.platform != get_windows_system4xi(): #WINDOWS324XI dragonized 

    # ---------------------------------------------------------------------------------------------
    # Non Windows
    # --------------------------------------------------------------------------------------------- 
    class2print = "popen4xi"
    method2print = "popen4xi"

    message2print = "{commenting" + "; Popen4Xi-commenting-by-popen4xi: Linux etc i.e. NOT Windows such as Vista 64 " + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
         

    # ---------------------------------------------------------------------------------------------
    # Non Windows 
    # --------------------------------------------------------------------------------------------- 
    
    exit = os._exit
    
    message2print = "{set" + "; exit-set-by-popen4xi to: " + str(exit)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
        
    
    # ---------------------------------------------------------------------------------------------
    # Non Windows 
    # ---------------------------------------------------------------------------------------------             
    duplicate = os.dup
    
    message2print = "{set" + "; duplicate-set-by-popen4xi to: " + str(duplicate)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
    
    # ---------------------------------------------------------------------------------------------
    #  Non Windows
    # ---------------------------------------------------------------------------------------------      
    close = os.close      

    message2print = "{set" + "; close-set-by-popen4xi to: " + str(close)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    
    # =================================================================================================
    #<class4xi class = "Popen4Xi" base = "object" child = "None" synopsis = "similar to the one from _winapi with constructor taking a process object as its argument" usage = "
    #
    # assert_spawning4xi.py:
    #     if not Popen4Xi.thread_is_spawning():
    # authenticationstring4xi.py:
    #     if not Popen4Xi.thread_is_spawning():
    # baseproxy4xi.py:
    #     if Popen4Xi.thread_is_spawning():
    # current_process4xi.py
    #     self._popen = Popen4Xi(self)
    # mainprocess4xi.py:
    #     self._popen = Popen4Xi(self)
    # process4xi.py:
    #     if self._Popen is not None:              # self._Popen already exists
    #         Popen4Xi = self._Popen               # set class to equal existing one
    #     else:
    #         from popen4xi import Popen4Xi        # __init__ has set to None, use Popen4Xi
    #         self._popen = Popen4Xi(self)         # sets object self._popen
    # reduce_handle4xi.py:
    #     if Popen4Xi.thread_is_spawning():
    #
    #     return (None, Popen4Xi.duplicate_for_child(handle), True)
    #
    # semlock4xi.py:
    #     return (Popen4Xi.duplicate_for_child(sl.handle), sl4xi.kind, sl4xi.maxvalue)"
    #>
    #<note4xi note = "NON-Windows OS"/>
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # =================================================================================================
    class Popen4Xi(object): 
        '''
        Popen4Xi class: similar to the one from _winapi with constructor taking a process object as its argument
        '''
    
        # ---------------------------------------------------------------------------------------------
        # Public API of this class -  Non Windows
        # ---------------------------------------------------------------------------------------------    
        #<expose4xi expose = "
        _exposed_ = ('poll', 'wait', 'terminate',
                     'sense', 'reason', 'ace')          
        #"
        #/>    

        #                                        
        # ---------------------------------------------------------------------------------------------
        # sharable class variables -  Non Windows
        # ---------------------------------------------------------------------------------------------      
        #<share4xi share = " 
        _tls = _thread._local()          # Thread-local storage
        
        cf2xi = ConstantForking4Xi()     # Constants for forking     
        #"
        #/>
                
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "init-of-Popen4Xi" usage = "initializes Popen4Xi with (self, process_obj)">
        #<note4xi note = "NON-Windows OS"/>
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def __init__(self, process_obj):
            '''
            initializes Popen4Xi -  Non Windows
            '''        
            
            # -----------------------------------------------------------------------------------------
            # (Step 1/10)            
            # -----------------------------------------------------------------------------------------
            class2print = "Popen4Xi"
            method2print = "__init__"

            message2print = "{(Step 1/10a) started" + "; init-of-Popen4Xi-OSx-started: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
                                       
            message2print = "{(Step 1/10b) passed" + "; process-object-passed-to-init-of-Popen4Xi: " + str(process_obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            message2print = "{(Step 1/10c) borning" + "; popen2xi-of-Popen4Xi-OSx-borning: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            
            message2print = "{(Step 1/10d) inheriting" + "; object-inheriting-by-popen2xi-of-Popen4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                
            message2print = "{(Step 1/10e) sharing" + "; thread-local-storage-sharing-by-popen2xi-of-Popen4Xi: " + str(Popen4Xi._tls)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
                               
                     
                                   

            # -----------------------------------------------------------------------------------------
            # (Step 2/10) -  Non Windows            
            # -----------------------------------------------------------------------------------------            
            sys.stdout.flush()
            
            message2print = "{(Step 2/10) performed" + "; sys-stdout-flush-of-popen2xi-performed-by-init-of-Popen4Xi: " + str(sys.stdout.flush) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                  
                    
                    
            # -----------------------------------------------------------------------------------------
            # (Step 3/10) -  Non Windows            
            # -----------------------------------------------------------------------------------------            
            sys.stderr.flush()
            
            message2print = "{(Step 3/10) performed" + "; sys-stderr-flush-performed-by-init-of-Popen4Xi: " + str(sys.stderr.flush) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
               

               
            # -----------------------------------------------------------------------------------------
            # (Step 4/10) -  Non Windows            
            # -----------------------------------------------------------------------------------------            
            self.returncode = None
            
            message2print = "{(Step 4/10) set" + "; return-code-of-popen2xi-set-by-init-of-Popen4Xi to: " + str(self.returncode)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                

            # -----------------------------------------------------------------------------------------
            # (Step 5/10) -  Non Windows            
            # -----------------------------------------------------------------------------------------
            r2xi, w2xi = os.pipe()
            message2print = "{(Step 5a/10) set" + "; r2xi-set-by-init-of-Popen4Xi to: " + str(r2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            message2print = "{(Step 5b/10) set" + "; w2xi-set-by-init-of-Popen4Xi to: " + str(w2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            self.sentinel = r2xi
            message2print = "{(Step 5c/10) set" + "; sentinel-of-popen2xi-set-by-init-of-Popen4Xi to: " + str(self.sentinel)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            
            self.pid = os.fork()
            
            message2print = "{(Step 5d/10) set" + "; pid-of-popen2xi-set-by-init-of-Popen4Xi to: " + str(self.pid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
          
      
            
            if self.pid == 0:
            
                # -------------------------------------------------------------------------------------
                # (Step 6/10) -  Non Windows            
                # -------------------------------------------------------------------------------------
                os.close(r2xi)
                message2print = "{(Step 6a/10) performed" + "; close-performed-by-init-of-Popen4Xi: " + str(os.close) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                if 'random' in sys.modules:
                    import random
                    
                    message2print = "{(Step 6b/10) imported" + "; random-imported-by-init-of-Popen4Xi: " + str(random) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)         
                     
                  
                    random.seed()
                    
                    
                
                # -------------------------------------------------------------------------------------
                # (Step 7/10) -  Non Windows            
                # -------------------------------------------------------------------------------------                    
                message2print = "{(Step 7/10a) calling" + "; process-object-bootstrap-calling-by-init-of-Popen4Xi: " + str(process_obj._bootstrap) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                
                
                boot_code2xi = process_obj._bootstrap()
                
                message2print = "{(Step 7/10b) set" + "; boot-code2xi-set-by-init-of-Popen4Xi to: " + str(boot_code2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                                
                # -------------------------------------------------------------------------------------
                # (Step 9/10) -  Non Windows            
                # -------------------------------------------------------------------------------------                
                message2print = "{(Step 9/10) passing" + "; boot-code2xi-passing-to-os-exit-by-init-of-Popen4Xi: " + str(boot_code2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                os._exit(boot_code2xi)
            
            # -----------------------------------------------------------------------------------------    
            # (Step 10/10) -  Non Windows 
            # -----------------------------------------------------------------------------------------                 
            os.close(w2xi)
            message2print = "{(Step 10a/10) performed" + "; close-performed-by-init-of-Popen4Xi: " + str(os.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            Finalize4Xi(self, os.close, (r2xi,))
            message2print = "{(Step 10b/10) performed" + "; Finalize4Xi-performed-by-init-of-Popen4Xi: " + str(Finalize4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            message2print = "{(Step 10c/10) activated" + "; popen2xi-of-Popen4Xi-OSx-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                             
            
            message2print = "{(Step 10d/10) inherited" + "; object-inherited-by-popen2xi-of-Popen4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                
            message2print = "{(Step 10e/10) shared" + "; thread-local-storage-shared-by-popen2xi-of-Popen4Xi: " + str(Popen4Xi._tls)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
        
            message2print = "{(Step 10f/10) returning" + "; init-of-Popen4Xi-OSx-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
                            
                                    
        #</methodize4xi>
    
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "poll-of-Popen4Xi" usage = "polls with (self, flag=os.WNOHANG)">
        #<note4xi note = "NON-Windows OS"/>
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def poll(self, flag=os.WNOHANG):
            '''
            polls with (self, flag) 
            ''' 
            
            # -----------------------------------------------------------------------------------------
            # (Step 1/3) -  Non Windows  
            # -----------------------------------------------------------------------------------------
            class2print = "Popen4Xi"
            method2print = "poll"
            
               
            message2print = "{(Step 1/3a) started" + "; poll-of-Popen4Xi-OSx-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                                       
            message2print = "{(Step 1/3b) passed" + "; flag-passed-to-poll-of-Popen4Xi: " + str(flag) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 

            # -----------------------------------------------------------------------------------------
            # (Step 2/3) -  Non Windows           
            # -----------------------------------------------------------------------------------------           
            if self.returncode is None:
            
                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
                try:
                    pid, sts = os.waitpid(self.pid, flag)
                
                    message2print = "{(Step 2/3a) got" + "; pid-got-by-poll-of-Popen4Xi: " + str(pid)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                            
                 
                    message2print = "{(Step 2/3b) got" + "; sts-got-by-poll-of-Popen4Xi: " + str(sts)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                except os.error:
                    # Child process not yet created. See #1731717
                    # e.errno == errno.ECHILD == 10
                    message2print = "{(Step 2/3z) returning" + "; None-returning-by-poll-of-Popen4Xi: " + str(None) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)    
                    return None
                    
                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
                if pid == self.pid:
    
                    # -----------------------------------------------------------------------------    
                    # -  Non Windows
                    # ----------------------------------------------------------------------------- 
                
                    message2print = "{(Step 2/3c) passing" + "; sts-passing-to-os-WIFSIGNALED-by-poll-of-Popen4Xi: " + str(sts) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            

                    if os.WIFSIGNALED(sts):
    
                        # -------------------------------------------------------------------------    
                        # -  Non Windows
                        # ------------------------------------------------------------------------- 
                    
                        message2print = "{(Step 2/3d) passing" + "; sts-passing-to-os-WTERMSIG-by-poll-of-Popen4Xi: " + str(sts) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                        
                        self.returncode = -os.WTERMSIG(sts)
    
                        # -------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------- 
                        
                        message2print = "{(Step 2/3e) set" + "; return-code-of-popen2xi-set-by-poll-of-Popen4Xi to: " + str(self.returncode)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                
    
                    # -----------------------------------------------------------------------------    
                    # -  Non Windows
                    # ----------------------------------------------------------------------------- 
                
                    else:
    
                        # -------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------- 
                    
                        message2print = "{(Step 2/3f) passing" + "; sts-passing-to-os-WIFEXITED-by-poll-of-Popen4Xi: " + str(sts) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                        
                        assert os.WIFEXITED(sts)
    
                        # -------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------- 
                        
                        message2print = "{(Step 2/3g) asserted" + "; sts-asserted-by-poll-of-Popen4Xi: " + str(sts) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)      
                        
                        self.returncode = os.WEXITSTATUS(sts)
    
                        # -------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------- 
                        
                        message2print = "{(Step 2/3h) set" + "; return-code-of-popen2xi-set-by-poll-of-Popen4Xi to: " + str(self.returncode)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
                
                                        
            # -----------------------------------------------------------------------------------------
            # (Step 3/3) -  Non Windows            
            # -----------------------------------------------------------------------------------------                        
            message2print = "{(Step 3/3) returning" + "; return-code-of-popen2xi-returning-by-poll-of-Popen4Xi: " + str(self.returncode) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                        
                                                
            return self.returncode            
        #</methodize4xi>
                
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "wait-of-Popen4Xi" usage = "waits with (self, timeout)">
        #<note4xi note = "NON-Windows OS"/>
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def wait(self, timeout=None):
            '''
            Popen4Xi method wait: waits with (self, timeout)
            ''' 
            
            # -----------------------------------------------------------------------------------------
            # (Step 1/6) -  Non Windows           
            # -----------------------------------------------------------------------------------------
            class2print = "Popen4Xi"
            method2print = "wait"
            
               
            message2print = "{(Step 1/6a) started" + "; wait-of-Popen4Xi-OSx-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
    
            # -----------------------------------------------------------------------------------------
            # (Step 2/6) -  Non Windows           
            # -----------------------------------------------------------------------------------------            
            if timeout is None:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                message2print = "{(Step 2/6) passing" + "; 0-passing-to-poll-of-popen2xi-by-wait-of-Popen4Xi: " + str(0) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            

                return self.poll(0)
                
            # -----------------------------------------------------------------------------------------
            # (Step 3/6) -  Non Windows           
            # -----------------------------------------------------------------------------------------  
            timeout = omega2timeout4xi(class2print, method2print) 

            message2print = "{(Step 3/6a) set" + "; timeout-set-by-wait-of-Popen4Xi-OSx to: " + str(timeout)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
          
            deadline = time.time() + timeout
            
            message2print = "{(Step 3/6b) set" + "; deadline-set-by-wait-of-Popen4Xi to: " + str(deadline)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                            

                         
            # -----------------------------------------------------------------------------------------
            # (Step 4/6) -  Non Windows           
            # -----------------------------------------------------------------------------------------            
            delay = 0.0005
            
            message2print = "{(Step 4/6) set" + "; delay-set-by-wait-of-Popen4Xi to: " + str(delay)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                

                          
            # -----------------------------------------------------------------------------------------
            # (Step 5/6) -  Non Windows           
            # -----------------------------------------------------------------------------------------

                                    
            while 1:

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
            
                response2xi = self.poll()
                
                            
                message2print = "{(Step 5/6a) got" + "; response2xi-got-by-wait-of-Popen4Xi to: " + str(response2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
    
                if response2xi is not None:
                    break
                    

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
                    
                remaining = deadline - time.time()
                
                            
                message2print = "{(Step 5/6b) set" + "; remaining-set-by-wait-of-Popen4Xi to: " + str(remaining)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
    
                if remaining <= 0:
                    break

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
                
                delay = min(delay * 2, remaining, 0.05)
                            
                message2print = "{(Step 5/6c) set" + "; delay-set-by-wait-of-Popen4Xi to: " + str(delay)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
    
                message2print = "{(Step 5/6d) passing" + "; delay-passing-to-time-sleep-by-wait-of-Popen4Xi: " + str(delay) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
                                
                time.sleep(delay)

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/6) -  Non Windows           
            # -----------------------------------------------------------------------------------------                
            message2print = "{(Step 5/6) returning" + "; response2xi-returning-by-wait-of-Popen4Xi: " + str(response2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                        
            
            return response2xi            
        #</methodize4xi>
                    
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "terminate-of-Popen4Xi" usage = "terminates with (self)">
        #<note4xi note = "NON-Windows OS"/>
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def terminate(self):
            '''
            Popen4Xi method terminate: terminates with (self)
            ''' 
            
            # -----------------------------------------------------------------------------------------
            # (Step 1/3) -  Non Windows           
            # -----------------------------------------------------------------------------------------
            class2print = "Popen4Xi"
            method2print = "terminate"
            
               
            message2print = "{(Step 1/3) started" + "; terminate-of-Popen4Xi-OSx-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   

            
            # -----------------------------------------------------------------------------------------
            # (Step 2/3) -  Non Windows           
            # -----------------------------------------------------------------------------------------          
            if self.returncode is None:

                # ---------------------------------------------------------------------------------    
                #
                # ---------------------------------------------------------------------------------     
            
                try:
    
                    # -----------------------------------------------------------------------------    
                    # -  Non Windows
                    # ----------------------------------------------------------------------------- 
                
                    message2print = "{(Step 2/3a) passing" + "; pid-of-popen2xi-passing-to-os-kill-by-terminate-of-Popen4Xi: " + str(self.pid) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                
                    message2print = "{(Step 2/3b) passing" + "; signal-SIGTERM-passing-to-os-kill-by-terminate-of-Popen4Xi: " + str(signal.SIGTERM) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                   
    
                    # -----------------------------------------------------------------------------    
                    #
                    # ----------------------------------------------------------------------------- 
                           
                    os.kill(self.pid, signal.SIGTERM)
                    
                    message2print = "{(Step 2/3c) performed" + "; os-kill-performed-by-terminate-of-Popen4Xi: " + str(os.kill) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                                    

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
                    
                except OSError as e:
    
                    # -----------------------------------------------------------------------------    
                    #
                    # ----------------------------------------------------------------------------- 
                
                
                    if self.wait(timeout=0.1) is None:
    
                        # -------------------------------------------------------------------------    
                        #
                        # ------------------------------------------------------------------------- 
                    
                        message2print = "{(Step 2/3d) hit" + "; OSError-hit-by-terminate-of-Popen4Xi: " + " *** terminate-of-Popen4Xi (Step 2/3) *** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                            
                                            
                        
                        raise
    
                    # -----------------------------------------------------------------------------    
                    #
                    # ----------------------------------------------------------------------------- 

                # ---------------------------------------------------------------------------------    
                # -  Non Windows
                # ---------------------------------------------------------------------------------     
                        
            # -----------------------------------------------------------------------------------------
            # (Step 3/3) -  Non Windows           
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 3/3) returning" + "; terminate-of-Popen4Xi-OSx-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
                            
                                    
        #</methodize4xi>
                
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "thread-is-spawning-of-Popen4Xi" usage = "returns with no parameters">
        #<note4xi note = "NON-Windows OS"/>
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        @staticmethod
        def thread_is_spawning():
            '''
            Popen4Xi method thread_is_spawning: returns with no parameters
            '''         
            
            # -----------------------------------------------------------------------------------------
            # (Step 1/2 -  Non Windows)          
            # -----------------------------------------------------------------------------------------
            class2print = "Popen4Xi"
            method2print = "thread_is_spawning"
            
               
            message2print = "{(Step 1/2) started" + "; thread-is-spawning-of-Popen4Xi-OSx-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
  
         
            # -----------------------------------------------------------------------------------------
            # (Step 2/2) -  Non Windows           
            # -----------------------------------------------------------------------------------------                  
            message2print = "{(Step 2/2) returning" + "; false-returning-by-thread-is-spawning-of-Popen4Xi: " + str(False) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                          
            
            return False
        #</methodize4xi>
                                           
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "sense-of-Popen4Xi" usage = "senses Popen4Xi class objects">
        #<note4xi note = "NON-Windows OS"/>
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def sense(self):
            '''
            method sense: senses Popen4Xi class objects
            '''
            
            # -----------------------------------------------------------------------------------------
            # (Step 1/5) -  Non Windows          
            # -----------------------------------------------------------------------------------------
            
            class2print = 'Popen4Xi'
            method2print = 'sense'
            
               
            message2print = "{(Step 1/5) started" + "; sense-of-Popen4Xi-OSx-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
  
            
            # -----------------------------------------------------------------------------------------
            # (Step 2/5) -  Non Windows
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 2/5) sensing" + "; Popen4Xi-sensing-by-sense-of-Popen4Xi: " + str(Popen4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                      
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/5) -  Non Windows
            # -----------------------------------------------------------------------------------------        
            kb2xi = sense_object4xi(self)
            
            message2print = "{(Step 3/5) got" + "; knowledge-base2xi-got-by-sense-of-Popen4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
            # -----------------------------------------------------------------------------------------
            # (Step 4/5) add knowledge specific to this object
            # format example: kb2xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------         
        
            
            
          
            message2print = "{(Step 4/5) sensed" + "; Popen4Xi-sensed-by-sense-of-Popen4Xi: " + str(Popen4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/5) -  Non Windows
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 5/5) returning" + "; knowledge-base2xi-returning-by-sense-of-Popen4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                   
              
            return kb2xi
            
        #</methodize4xi>
          
    
            
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "reason-of-Popen4Xi" usage = "reasons Popen4Xi class objects">
        #<note4xi note = "NON-Windows OS"/>
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def reason(self):
            '''
            method reason: reasons Popen4Xi class objects
            '''
            
            # -----------------------------------------------------------------------------------------
            # (Step 1/5) -  Non Windows          
            # -----------------------------------------------------------------------------------------
                 
            
            class2print = 'Popen4Xi'
            method2print = 'reason'
    
               
            message2print = "{(Step 1/5) started" + "; reason-of-Popen4Xi-OSx-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
  
            
            # -----------------------------------------------------------------------------------------
            # (Step 2/5) -  Non Windows
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 2/5) reasoning" + "; Popen4Xi-reasoning-by-reason-of-Popen4Xi: " + str(Popen4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
            
            # -----------------------------------------------------------------------------------------
            #  (Step 3/5) get reasoned knowledge for this object -  Non Windows
            # -----------------------------------------------------------------------------------------         
            kb4xi = reason_object4xi(self)
            
            message2print = "{(Step 3/5) got" + "; knowledge-base4xi-got-by-by-reason-of-Popen4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            # -----------------------------------------------------------------------------------------
            #  (Step 4/5) add knowledge specific to this object
            # format example: kb4xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------          
            
            
              
            
            message2print = "{(Step 4/5) reasoning" + "; Popen4Xi-reasoning-by-reason-of-Popen4Xi: " + str(Popen4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/5) -  Non Windows
            # -----------------------------------------------------------------------------------------        
            message2print = "{(Step 5/5) returning" + "; knowledge-base4xi-returning-by-reason-of-Popen4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                              
              
            return kb4xi
            
        #</methodize4xi>
             
    
        
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "ace-of-Popen4Xi" usage = "aces Popen4Xi class objects">
        #<note4xi note = "NON-Windows OS"/>
        #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def ace(self):
            '''
            method ace: aces Popen4Xi class objects 
            '''
 
            
            # -----------------------------------------------------------------------------------------
            # (Step 1/5) -  Non Windows          
            # -----------------------------------------------------------------------------------------
                
            
            class2print = 'Popen4Xi'
            method2print = 'ace'
            
               
            message2print = "{(Step 1/5) started" + "; ace-of-Popen4Xi-OSx-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
  
            
            # -----------------------------------------------------------------------------------------
            # (Step 2/5) -  Non Windows
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 2/5) acing" + "; Popen4Xi-acing-by-ace-of-Popen4Xi: " + str(Popen4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/5) get aced knowledge for this object -  Non Windows
            # -----------------------------------------------------------------------------------------        
            kb8xi = ace_object4xi(self)  
            
            message2print = "{(Step 3/5)got" + "; knowledge-base8xi-got-by-ace-of-Popen4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    
    
            # -----------------------------------------------------------------------------------------
            # (Step 4/5) add knowledge specific to this object
            # format example: kb8xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------              
            
              
            message2print = "{(Step 4/5) acing" + "; Popen4Xi-acing-by-ace-of-Popen4Xi: " + str(Popen4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/5) -  Non Windows
            # -----------------------------------------------------------------------------------------        
            message2print = "{(Step 5/5) returning" + "; knowledge-base8xi-returning-by-ace-of-Popen4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                   
            
            return kb8xi
        #</methodize4xi>
#</class4xi>
else:
    # =============================================================================================
    # sys.platform == WINDOWS324XI  <--- VISTA 64-BIT HERE   
    # =============================================================================================

    #<note4xi note = "
    # dragonized: this code was moved here; original code belonged to nowhere!
    # _python_exe is the assumed path to the python executable.
    # People embedding Python want to modify it.
    #"
    #/>
    
    # -------------------------------------------------------------------------------------------------
    # Global constants - Windows                            
    # ------------------------------------------------------------------------------------------------
    #<globalize4xi globalization = "

    #"
    #/>
     
    TERMINATE = 0x10000
    WINEXE = (sys.platform == 'win32' and getattr(sys, 'frozen', False))
    WINSERVICE = sys.executable.lower().endswith("pythonservice.exe")
    close = _winapi.CloseHandle
    
    if WINSERVICE:
        _python_exe = os.path.join(sys.exec_prefix, 'python.exe')
    else:
        _python_exe = sys.executable

    # ---------------------------------------------------------------------------------------------
    #<function4xi function = "dump-of-Popen4Xi" usage = "
    # (obj, file, protocol=None)
    #"> 
    #<note4xi note = "Windows OS"/>
    #<state4xi status = "ACK4tagging; ACK4hello_world4xi.py; NACK4moduletests; NACK4systemtest"/>
    # ---------------------------------------------------------------------------------------------
    def dump(obj, file, protocol=None):
            
        # -----------------------------------------------------------------------------------------
        # (Step 1/4) - Windows           
        # -----------------------------------------------------------------------------------------
        
        class2print = "dump"
        method2print = "dump"
        
               
        message2print = "{(Step 1/4a) started" + "; dump-of-Popen4Xi-Windows-started: " + str(dump)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
                                                                                                                    # HW:  <function dump at 0x0000000003A85048>; time: 01:41:17 02/18/11 Pacific Standard Time}
                                       
        message2print = "{(Step 1/4b) passed" + "; object-passed-to-dump-of-Popen4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                                                                                                                    # HW: {'authkey': b'\x89\xf2\xa6XaQPhH\xba\xa3\xae\x0b\xd2\x19\xa63H*\xb6d\x13\x81\x0e\x1f\xc0\xf1U\xf7\x01.\x18', 
                                                                                                                    #     'sys_path': ['E:\\Sw4Xi\\Python32\\Lib\\multiprocessing4xi', 'd:\\Python32', 'D:\\Windows\\system32\\python32.zip', 'd:\\Python32\\DLLs', 'd:\\Python32\\lib', 'd:\\Python32\\lib\\site-packages'], 
                                                                                                                    #         'name': 'Process4Xi-1', 
                                                                                                                    #     'orig_dir': 'E:\\Sw4Xi\\Python32\\Lib\\multiprocessing4xi', 
                                                                                                                    #     'sys_argv': ['hello_world4xi.py'], 
                                                                                                                    #    'main_path': 'E:\\Sw4Xi\\Python32\\Lib\\multiprocessing4xi\\hello_world4xi.py', 
                                                                                                                    #'log_to_stderr': True}; time: 01:41:17 02/18/11 Pacific Standard Time}
   
        
                               
        message2print = "{(Step 1/4c) passed" + "; file-passed-to-dump-of-Popen4Xi: " + str(file) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: 1. and 2. : <_io.BufferedWriter name=4>
                                       
        message2print = "{(Step 1/4d) passed" + "; protocol-passed-to-dump-of-Popen4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: 1. and 2. : 3         
        
        

        # --------------------------------------------------------------------------------------------------
        # see # test_proxy_mp_xi4.py: Step 4/12 create three managers
        # --------------------------------------------------------------------------------------------------
                
        #1 {'authkey': "b'\\xf4n\\xebg\\n\\x1a\\x14\\xe2(\\xe0\\xa2\\x10\\xf4\\xda\\x8d\\xfcwtC\\xb2\\xe8\\x8b\\x01^\\xa8\\x9al_\\xcc\\xdcM\\xbf'", 'sys_path': ['e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'c:\\sw\\Python30', 'c:\\sw\\Python30\\python30.zip', 'c:\\sw\\Python30\\DLLs', 'c:\\sw\\Python30\\lib', 'c:\\sw\\Python30\\lib\\plat-win', 'c:\\sw\\Python30\\lib\\site-packages'],
        #      'name': 'SyncManager4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'sys_argv': ['test_proxy_mp_xi4.py'], 'main_path': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py', 'log_to_stderr': True}; file: <io.BufferedWriter object at 0x000000000389B5C0>; protocol: 3; time: 11:22:38 07/03/09 FLE Daylight Time}
        #2 <Process4Xi(SyncManager4Xi-1, initial)>; file: <io.BufferedWriter object at 0x000000000389B5C0>; protocol: 3; time: 11:22:38 07/03/09 FLE Daylight Time}

        #3 {'authkey': "b'\\xf4n\\xebg\\n\\x1a\\x14\\xe2(\\xe0\\xa2\\x10\\xf4\\xda\\x8d\\xfcwtC\\xb2\\xe8\\x8b\\x01^\\xa8\\x9al_\\xcc\\xdcM\\xbf'", 'sys_path': ['e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'c:\\sw\\Python30', 'c:\\sw\\Python30\\python30.zip', 'c:\\sw\\Python30\\DLLs', 'c:\\sw\\Python30\\lib', 'c:\\sw\\Python30\\lib\\plat-win', 'c:\\sw\\Python30\\lib\\site-packages'],
        #      'name': 'SyncManager4Xi-2', 'orig_dir': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'sys_argv': ['test_proxy_mp_xi4.py'], 'main_path': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py', 'log_to_stderr': True}; file: <io.BufferedWriter object at 0x000000000389B710>; protocol: 3; time: 11:22:38 07/03/09 FLE Daylight Time}
        #4 <Process4Xi(SyncManager4Xi-2, initial)>; file: <io.BufferedWriter object at 0x000000000389B710>; protocol: 3; time: 11:22:38 07/03/09 FLE Daylight Time}

        
        #5 {'authkey': "b'\\xf4n\\xebg\\n\\x1a\\x14\\xe2(\\xe0\\xa2\\x10\\xf4\\xda\\x8d\\xfcwtC\\xb2\\xe8\\x8b\\x01^\\xa8\\x9al_\\xcc\\xdcM\\xbf'", 'sys_path': ['e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'c:\\sw\\Python30', 'c:\\sw\\Python30\\python30.zip', 'c:\\sw\\Python30\\DLLs', 'c:\\sw\\Python30\\lib', 'c:\\sw\\Python30\\lib\\plat-win', 'c:\\sw\\Python30\\lib\\site-packages'], 
        #      'name': 'SyncManager4Xi-3', 'orig_dir': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'sys_argv': ['test_proxy_mp_xi4.py'], 'main_path': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py', 'log_to_stderr': True}; file: <io.BufferedWriter object at 0x000000000389B780>; protocol: 3; time: 11:22:38 07/03/09 FLE Daylight Time}
        #6 <Process4Xi(SyncManager4Xi-3, initial)>; file: <io.BufferedWriter object at 0x000000000389B780>; protocol: 3; time: 11:22:38 07/03/09 FLE Daylight Time}

        
        # --------------------------------------------------------------------------------------------------
        # see Step 11/12g: creating process2ctypes object with Process4Xi giving modify2xi function and args 
        # --------------------------------------------------------------------------------------------------
                
        #7 {'authkey': "b'\\xf4n\\xebg\\n\\x1a\\x14\\xe2(\\xe0\\xa2\\x10\\xf4\\xda\\x8d\\xfcwtC\\xb2\\xe8\\x8b\\x01^\\xa8\\x9al_\\xcc\\xdcM\\xbf'", 'sys_path': ['e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'c:\\sw\\Python30', 'c:\\sw\\Python30\\python30.zip', 'c:\\sw\\Python30\\DLLs', 'c:\\sw\\Python30\\lib', 'c:\\sw\\Python30\\lib\\plat-win', 'c:\\sw\\Python30\\lib\\site-packages'], 
        #      'name': 'Process4Xi-4', 'orig_dir': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'sys_argv': ['test_proxy_mp_xi4.py'], 'main_path': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py', 'log_to_stderr': True}; file: <io.BufferedWriter object at 0x000000000389B8D0>; protocol: 3; time: 11:22:39 07/03/09 FLE Daylight Time}
        #8 <Process4Xi(Process4Xi-4, initial)>; file: <io.BufferedWriter object at 0x000000000389B8D0>; protocol: 3; time: 11:22:39 07/03/09 FLE Daylight Time}

        # --------------------------------------------------------------------------------------------------
        # Step 12/12g: creating process3xyz object with Process4Xi
        # --------------------------------------------------------------------------------------------------        
        #9{'authkey': "b'\\xf4n\\xebg\\n\\x1a\\x14\\xe2(\\xe0\\xa2\\x10\\xf4\\xda\\x8d\\xfcwtC\\xb2\\xe8\\x8b\\x01^\\xa8\\x9al_\\xcc\\xdcM\\xbf'", 'sys_path': ['e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'c:\\sw\\Python30', 'c:\\sw\\Python30\\python30.zip', 'c:\\sw\\Python30\\DLLs', 'c:\\sw\\Python30\\lib', 'c:\\sw\\Python30\\lib\\plat-win', 'c:\\sw\\Python30\\lib\\site-packages'], 
        #     'name': 'Process4Xi-5', 'orig_dir': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'sys_argv': ['test_proxy_mp_xi4.py'], 'main_path': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py', 'log_to_stderr': True}; file: <io.BufferedWriter object at 0x000000000389BCC0>; protocol: 3; time: 11:22:39 07/03/09 FLE Daylight Time}
        #10 <Process4Xi(Process4Xi-5, initial)>; file: <io.BufferedWriter object at 0x000000000389BCC0>; protocol: 3; time: 11:22:39 07/03/09 FLE Daylight Time}
        #
        #hw obj:
        #{'authkey': "b'\\xe8\\xaf\\xcb\\xf6\\xa0\\xc1[-\\xdf\\x17\\x85\\xd4r\\xd5\\x106\\xcd\\x1a\\xee\\xca\\x19\\x9fE\\x88*0\\xf2\\x1a\\x11TC\\xa5'", 
        #'sys_path': ['e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'c:\\sw\\Python30', 'c:\\sw\\Python30\\python30.zip', 'c:\\sw\\Python30\\DLLs', 'c:\\sw\\Python30\\lib', 'c:\\sw\\Python30\\lib\\plat-win', 'c:\\sw\\Python30\\lib\\site-packages'], 
        #    'name': 'Process4Xi-1', 
        #'orig_dir': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 
        #'sys_argv': ['hello_world4xi.py'], 'main_path': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi\\hello_world4xi.py', 'log_to_stderr': True}; 
        #      file: <io.BufferedWriter object at 0x000000000382AB70>; protocol: 3; time: 14:55:25 08/28/09 FLE Daylight Time}

        #{ get_preparation_data4xi : get_preparation_data4xi ; StepOriginal return forking.get_preparation_data(name) ;time: 07:30:39 09/03/09 FLE Daylight Time}
        # { dump : dump ; Popen4Xi function dump parameter obj: 
        #       'authkey': b'R\xb1\xb6\xba\x85\r\xef8\xf0\xfc\x8e\xb4\xc1\xc0\xf34^\x08\xd8\xbaAE\xd7nK\xd6\xd6v%\xc3\xae\xb6', 
        #      'sys_path': ['e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 'c:\\sw\\Python30', 'c:\\sw\\Python30\\python30.zip', 'c:\\sw\\Python30\\DLLs', 'c:\\sw\\Python30\\lib', 'c:\\sw\\Python30\\lib\\plat-win', 'c:\\sw\\Python30\\lib\\site-packages'], 
        #          'name': 'Process4Xi-3', 
        #      'orig_dir': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi', 
        #      'sys_argv': ['hello_world4xi.py'], 
        #     'main_path': 'e:\\Sw4Xi\\Python30\\Lib\\multiprocessing4xi\\hello_world4xi.py', 
        # 'log_to_stderr': False}; file: <io.BufferedWriter object at 0x000000000387B940>; protocol: 3; time: 07:30:39 09/03/09 FLE Daylight Time}

        
        
         
        
        # -------------------------------------------------------------------------------------    
        # (Step 2/4) - Windows 
        # -------------------------------------------------------------------------------------                 
        if examine4xi(class2print, method2print):
        
            # ---------------------------------------------------------------------------------    
            # StepOriginal
            # ---------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) passing" + "; file-passing-to-ForkingPickler-by-dump-of-Popen4Xi: " + str(file) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(StepOriginal) passing" + "; protocol-passing-to-ForkingPickler-by-dump-of-Popen4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            message2print = "{(StepOriginal) passing" + "; object-passing-to-ForkingPickler-by-dump-of-Popen4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                             
            # -----------------------------------------------------------------------------------------  
                                              
            forking.ForkingPickler(file, protocol).dump(obj)
            
            # -----------------------------------------------------------------------------------------  
            message2print = "{(StepOriginal) performed" + "; ForkingPickler-performed-by-dump-of-Popen4Xi: " + str(forking.ForkingPickler) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                
                    
        else:
        
            # -----------------------------------------------------------------------------------------
            # (Step 3/4) - Windows           
            # -----------------------------------------------------------------------------------------    
            message2print = "{(Step 3/4a) passing" + "; file-passing-to-ForkingPickler4Xi-by-dump-of-Popen4Xi: " + str(file) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #HW: <_io.BufferedWriter name=4>
                
            message2print = "{(Step 3/4b) passing" + "; protocol-passing-to-ForkingPickler4Xi-by-dump-of-Popen4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #HW: 3
            
            message2print = "{(Step 3/4c) passing" + "; object-passing-to-ForkingPickler4Xi-by-dump-of-Popen4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                      
            #HW: 1. { 'authkey': "b'\\xe94D\\xe6\\x06`\\xf1\\xe5\\xf3\\x19\\xe5\\x1b\\xd5a\\xa6\\x982@\\xc0W\\xbd\\xca\\x9aY[^\\x8a\\xcf\\xd9\\xc6\\x90`'", 
            #        'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'], 
            #            'name': 'Process4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 
            #        'sys_argv': ['hello_world4xi.py'], 
            #       'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py', 
            #   'log_to_stderr': True}
            #   ; time: 22:37:42 03/15/10 FLE Standard Time}

            
            #HW: 2. <Process4Xi(Process4Xi-1, initial)>
            
            # -----------------------------------------------------------------------------------------  
            
            ForkingPickler4Xi(file, protocol).dump(obj) # embeds now the function MakeProxyType4xi  #to be solved the latest
            
            # -----------------------------------------------------------------------------------------  

            message2print = "{(Step 3/4d) performed" + "; ForkingPickler4Xi-performed-by-dump-of-Popen4Xi: " + str(ForkingPickler4Xi) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)             
            #HW: 1. and 2. :<class 'forkingpickler4xi.ForkingPickler4Xi'>              
            
            # -----------------------------------------------------------------------------------------
            # (Step 4/4) - Windows           
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(Step 4/4) returning" + "; dump-of-Popen4Xi-Windows-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #HW: 22:57:09 03/15/10 FLE Standard Time            
                
                        
    #</function4xi>            
     
        
    
        
    # ---------------------------------------------------------------------------------------------
    #<function4xi function = "set-executable-of-Popen4Xi" usage = "
    # setting executable; __init__.py is NOT used to add this function public
    #                     instead it is defined as public API of this module
    #"> 
    #<note4xi note = "Windows OS"/>
    #<state4xi status = "ACK4tagging; ACK4hello_world4xi.py; NACK4moduletests; NACK4systemtest"/>
    # ---------------------------------------------------------------------------------------------
    def set_executable(exe):

        # ----------------------------------------------------------------------------------------
        # Global constants - Windows                             
        # ----------------------------------------------------------------------------------------
        #<globalize4xi globalization = "
        global _python_exe
        #"
        #/>
             
        # -----------------------------------------------------------------------------------------
        # (Step 1/3) - Windows           
        # -----------------------------------------------------------------------------------------
    
        class2print = "set_executable"   #do NOT define class here because this is an embedded function
        method2print = "set_executable"  #OK in print4xi.py and in hello_world4xi.py      
        
               
        message2print = "{(Step 1/3a) started" + "; set-executable-of-Popen4Xi-Windows-started: " + str(set_executable)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
      
                                       
        message2print = "{(Step 1/3b) passed" + "; exe-passed-to-set-executable-of-Popen4Xi: " + str(exe) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
           

        
        # -----------------------------------------------------------------------------------------
        # (Step 2/3) - Windows           
        # -----------------------------------------------------------------------------------------         
        _python_exe = exe

        message2print = "{(Step 2/3) set" + "; python-exe-set-by-set-executable-of-Popen4Xi to: " + str(_python_exe)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                
                     
                
        # -----------------------------------------------------------------------------------------
        # (Step 3/3) - Windows           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/3) returning" + "; set-executable-of-Popen4Xi-Windows-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
           
    #</function4xi>        
    
    # ---------------------------------------------------------------------------------------------
    #<function4xi function = "duplicate-of-Popen4Xi" usage = "
    # (handle, target_process=None, inheritable=False)
    #"> 
    #<note4xi note = "Windows OS"/>
    #<state4xi status = "ACK4tagging; ACK4hello_world4xi.py; NACK4moduletests; NACK4systemtest"/>
    # ---------------------------------------------------------------------------------------------
    def duplicate(handle, target_process=None, inheritable=False):
        
        
        # -------------------------------------------------------------------------------------    
        # (Step 1/3) - Windows   
        # -------------------------------------------------------------------------------------                 
        class2print = "duplicate"
        method2print = "duplicate"        
        
               
        message2print = "{(Step 1/3a) started" + "; duplicate-of-Popen4Xi-Windows-started: " + str(duplicate)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HW: 20:31:30 01/13/10 FLE Standard Time
        #PX: 16:32:57 04/06/10 FLE Daylight Time
                                       
        message2print = "{(Step 1/3b) passed" + "; handle-passed-to-duplicate-of-Popen4Xi: " + str(handle) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: 216
        #PX: 428
                                       
        message2print = "{(Step 1/3c) passed" + "; target-process-passed-to-duplicate-of-Popen4Xi: " + str(target_process) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: None
        #PX: None
                                               
        message2print = "{(Step 1/3d) passed" + "; inheritable-passed-to-duplicate-of-Popen4Xi: " + str(inheritable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: True
        #PX: True
        
        # -------------------------------------------------------------------------------------    
        # - Windows 
        # -------------------------------------------------------------------------------------                 
        if examine4xi(class2print, method2print):
        
            # ---------------------------------------------------------------------------------    
            # StepOriginal
            # ---------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) passing" + "; handle-passing-to-forking-duplicate-by-duplicate-of-Popen4Xi: " + str(handle) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
            message2print = "{(StepOriginal) passing" + "; target-process-passing-to-forking-duplicate-by-duplicate-of-Popen4Xi: " + str(target_process) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
            
            message2print = "{(StepOriginal) passing" + "; inheritable-passing-to-forking-duplicate-by-duplicate-of-Popen4Xi: " + str(inheritable) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            message2print = "{(StepOriginal) returning" + "; forking.duplicate-function-returning-by-duplicate-of-Popen4Xi: " + str(forking.duplicate) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                                         
            
            return forking.duplicate(handle, target_process, inheritable)
            

        else:
                            
            # -----------------------------------------------------------------------------------------
            # (Step 2/3) - Windows            
            # -----------------------------------------------------------------------------------------  
            if target_process is None:
               
                message2print = "{(Step 2/3a) calling" + "; _winapi.GetCurrentProcess-calling-by-duplicate-of-Popen4Xi: " + str(_winapi.GetCurrentProcess) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                #HW, #PX: <built-in function GetCurrentProcess>                
                                
                
                target_process = _winapi.GetCurrentProcess()
                
                message2print = "{(Step 2/3b) set" + "; target-process-set-by-duplicate-of-Popen4Xi to: " + str(target_process)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #HW, #PX: 
                

                
            # -----------------------------------------------------------------------------------------
            # (Step 3/3) - Windows           
            # -----------------------------------------------------------------------------------------                
            message2print = "{(Step 3/3a) passing" + "; inheritable-passing-to-_winapi.DuplicateHandle-by-duplicate-of-Popen4Xi: " + str(inheritable) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #HW: True
            
            message2print = "{(Step 3/3b) passing" + "; handle-passing-to-_winapi.DuplicateHandle-by-duplicate-of-Popen4Xi: " + str(handle) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                             
            #HW: 216
            #PX: 428
            
            message2print = "{(Step 3/3c) passing" + "; target_process-passing-to-_winapi.DuplicateHandle-by-duplicate-of-Popen4Xi: " + str(target_process) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                             
            #HW, #PX: 
            
                
            message2print = "{(Step 3/3d) returning" + "; _winapi.DuplicateHandle-function-returning-by-duplicate-of-Popen4Xi: " + str(_winapi.DuplicateHandle) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            #HW, #PX: <built-in function DuplicateHandle>
                             
            
            return _winapi.DuplicateHandle(
            _winapi.GetCurrentProcess(), handle, target_process,
            0, inheritable, _winapi.DUPLICATE_SAME_ACCESS
            )
    #</function4xi>
 
    # =============================================================================================
    #<class4xi class = "Popen4Xi" base = "object" child = "None" synopsis = "Start a _winapi to run the code of a process object" usage = "
    #    Process4Xi:
    #        start: 
    #            Step 5/7a: checks self._Popen: None; 
    #            Step 5/7c: imported Popen4Xi: <class 'popen4xi.Popen4Xi'>;
    #"
    #>
    #<note4xi note = "Windows OS"/>    
    #<state4xi status = "ACK4tagging; ACK4hello_world4xi.py; NACK4moduletests; NACK4systemtest"/>
    # =============================================================================================
    class Popen4Xi(object):
        '''
        Start a _winapi to run the code of a process object
        '''
        
        # ---------------------------------------------------------------------------------------------
        # Public API of this class - Windows 
        # ---------------------------------------------------------------------------------------------    
        #<expose4xi expose = " 
        _exposed_ = ('poll', 'wait', 'terminate',
                     'sense', 'reason', 'ace')                      
      
        #"
        #/>

        #                                        
        # ---------------------------------------------------------------------------------------------
        # sharable class variables - Windows 
        # ---------------------------------------------------------------------------------------------      
        #<share4xi share = " 
        _tls = _thread._local()          # thread local storage
        
        cf2xi = ConstantForking4Xi()     # Constants for forking    
        
        #"
        #/>
        
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "init-of-Popen4Xi" usage = "initializes Popen4Xi">
        #<note4xi note = "Windows; Vista 64-bit"/>
        #<state4xi status = "ACK4tagging; ACK4hello_world4xi.py; NACK4moduletests; NACK4systemtest"/>
        # -----------------------------------------------------------------------------------------
        def __init__(self, process_obj):  
            '''
            initializes Popen4Xi
            '''
            
            # -------------------------------------------------------------------------------------
            # Global constants - Windows                             
            # -------------------------------------------------------------------------------------
            #<globalize4xi globalization = "
            global _python_exe
            #"
            #/>
                  
         
            # -------------------------------------------------------------------------------------
            # Step 0/8:  - Windows 
            # -------------------------------------------------------------------------------------    
            class2print = "Popen4Xi"
            method2print = "__init__" 
               
            message2print = "{(Step 0/8a) started" + "; init-of-Popen4Xi-Windows-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 10:59:28 01/21/13 Pacific Standard Time
                                       
            message2print = "{(Step 0/8b) borning" + "; popen2xi-of-Popen4Xi-Windows-borning: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 10:59:28 01/21/13 Pacific Standard Time
            
            message2print = "{(Step 0/8c) passed" + "; process-object-passed-to-init-of-Popen4Xi-Windows: " + str(process_obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <Process4Xi(Process4Xi-1, initial)>
            
            message2print = "{(Step 0/8d) inheriting" + "; object-inheriting-by-popen2xi-of-Popen4Xi-Windows: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # <class 'object'>
               
            message2print = "{(Step 0/8e) sharing" + "; thread-local-storage-sharing-by-popen2xi-of-Popen4Xi-Windows: " + str(Popen4Xi._tls)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            # <_thread._local object at 0x000000000343AEB8>
                                        
            # -------------------------------------------------------------------------------------    
            # - Windows 
            # -------------------------------------------------------------------------------------                
            if examine4xi(class2print, method2print):
        
                # ---------------------------------------------------------------------------------    
                # StepOriginal OK - Windows 
                # ---------------------------------------------------------------------------------             
                message2print = "{(StepOriginal) passing" + "; process-object-passing-to-Popen-init-by-init-of-Popen4Xi-Windows: " + str(process_obj) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(StepOriginal) calling" + "; Popen-init-calling-by-init-of-Popen4Xi-Windows: " + str(Popen.__init__) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                # 09:37:10 09/03/09 FLE Daylight Time}
                
                Popen.__init__(self, process_obj)
                
            else:
                
                # -------------------------------------------------------------------------------------
                # (Step 1/8): - Windows: CREATE PIPE FOR COMMUNICATION WITH CHILD 
                # -------------------------------------------------------------------------------------
                message2print = "{(Step 1/8a) calling" + "; os-pipe-calling-by-init-of-Popen4Xi-Windows: " + str(os.pipe) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                # <built-in function pipe>
                          
                rfd2xi, wfd2xi = os.pipe() 

                message2print = "{(Step 1/8b) got" + "; read-file-descriptor2xi-from-os-pipe-got-by-init-of-Popen4Xi-Windows: " + str(rfd2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 3
                
                message2print = "{(Step 1/8c) got" + "; write-file-descriptor2xi-from-os-pipe-got-by-init-of-Popen4Xi-Windows: " + str(wfd2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 4              
                
                # -------------------------------------------------------------------------------------
                # (Step 2/8): - Windows  
                #         http://bugs.python.org/issue4708 
                #         http://www.gossamer-threads.com/lists/python/bugs/709902
                #         http://code.google.com/p/python-multiprocessing/source/browse/trunk/Lib/multiprocessing/forking.py?spec=svn25&r=25
                #         http://bugs.python.org/issue4284; download MSVCRT for 64-bit Vista
                # -------------------------------------------------------------------------------------  
                # { Popen4Xi : __init__ ; Step 2/8a getting handle for dublicate function by calling msvcrt time: 11:22:38 07/03/09 FLE Daylight Time}
                
                message2print = "{(Step 2/8a) passing" + "; read-file-descriptor2xi-from-os-pipe-passing-to-msvcrt-get-osfhandle-by-init-of-Popen4Xi-Windows: " + str(rfd2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 3
                                
                message2print = "{(Step 2/8b) calling" + "; msvcrt-get-osfhandle-calling-by-init-of-Popen4Xi-Windows: " + str(msvcrt.get_osfhandle) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                # <built-in function get_osfhandle>              
                
                # -------------------------------------------------------------------------------------
                # (Step 2/8) - Windows: GET FILE HANDLE FIRST
                # ------------------------------------------------------------------------------------- 
      
                fhandle2xi = msvcrt.get_osfhandle(rfd2xi)
                
                message2print = "{(Step 2/8c) got" + "; file-handle2xi-got-by-init-of-Popen4Xi-Windows: " + str(fhandle2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 256
                
                message2print = "{(Step 2/8d) passing" + "; file-handle2xi-passing-to-duplicate-by-init-of-Popen4Xi-Windows: " + str(fhandle2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 256
                
                message2print = "{(Step 2/8e) passing" + "; None-passing-to-duplicate-by-init-of-Popen4Xi-Windows: " + str(None) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # None
                
                message2print = "{(Step 2/8f) passing" + "; true-passing-to-duplicate-by-init-of-Popen4Xi-Windows: " + str(True) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # True
                
                # -------------------------------------------------------------------------------------
                # (Step 2/8) - Windows: GET HANDLE FOR READ END OF THE PIPE AND MAKE IT INHERITABLE 
                # ------------------------------------------------------------------------------------- 
                      
                rhandle2xi = duplicate(fhandle2xi, target_process=None, inheritable=True)   # get handle for read end of the pipe and make it inheritable
                                                                                            
                message2print = "{(Step 2/8g) got" + "; read-handle2xi-from-duplicate-got-by-init-of-Popen4Xi-Windows: " + str(rhandle2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 268
                
                # -------------------------------------------------------------------------------------
                # (Step 2/8) - Windows: CLOSE WITH FILE HANDLER  
                # ------------------------------------------------------------------------------------- 
                            
                message2print = "{(Step 2/8h) passing" + "; read-file-descriptor2xi-from-os-pipe-passing-to-os-close-by-init-of-Popen4Xi-Windows: " + str(rfd2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 3            

                os.close(rfd2xi)
                
                message2print = "{(Step 2/8j) performed" + "; os-close-performed-by-init-of-Popen4Xi-Windows: " + str(os.close) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # <built-in function close>
                
                
                # -------------------------------------------------------------------------------------
                # (Step 3/8) - Windows: START PROCESS USING HANDLE FOR READ END OF THE PIPE   
                # -------------------------------------------------------------------------------------
                message2print = "{(Step 3/8a) passing" + "; read-handle2xi-from-duplicate-passing-to-cmd-by-init-of-Popen4Xi-Windows: " + str(rhandle2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 268            
    
                cmd2xi = get_command_line() + [rhandle2xi]
                
             
                message2print = "{(Step 3/8b) set" + "; command-set-by-init-of-Popen4Xi to: " + str(cmd2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # ['d:\\Python33\\python.exe', '-c', 'from popen4xi import main', '--multiprocessing-fork', 268]
                
                # -----------------------------------------------------------------------------------------    
                # (Step 3/8c) - Windows: IMPORT MAIN FROM POPEN4XI
                # -----------------------------------------------------------------------------------------    
                  
                cmd2xi = ' '.join('"%s"' % x for x in cmd2xi)
                
                message2print = "{(Step 3/8c) upgraded" + "; command-upgraded-by-init-of-Popen4Xi-Windows: " + str(cmd2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # "d:\Python33\python.exe" "-c" "from popen4xi import main" "--multiprocessing-fork" "268"
                
                
                # -----------------------------------------------------------------------------------------    
                # (Step 3/8d) - Windows 
                # -----------------------------------------------------------------------------------------    
                message2print = "{(Step 3/8d1) passing" + "; python-exe-passing-to-_winapi-CreateProcess-by-init-of-Popen4Xi-Windows: " + str(_python_exe) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # d:\Python33\python.exe
                
                message2print = "{(Step 3/8d2) passing" + "; command-passing-to-_winapi-CreateProcess-by-init-of-Popen4Xi-Windows: " + str(cmd2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                     
                # "d:\Python33\python.exe" "-c" "from popen4xi import main" "--multiprocessing-fork" "268"
                
                try:           
                    hp2xi, ht2xi, pid2xi, tid2xi = _winapi.CreateProcess(                       #Creates a new process and its primary thread
                        _python_exe, cmd2xi, None, None, 1, 0, None, None, None
                        )
    
                    # -----------------------------------------------------------------------------------------    
                    # References:
                    #             http://groups.google.com/group/comp.lang.python/browse_thread/thread/5d761356383feb5b
                    #             
                    # -----------------------------------------------------------------------------------------    
                
                    # -----------------------------------------------------------------------------------------    
                    # (Step 3/8e) - Windows HANDLE HP2XI
                    # -----------------------------------------------------------------------------------------    
 
                    message2print = "{(Step 3/8e) got" + "; handle-hp2xi-from-CreateProcess-got-by-init-of-Popen4Xi-Windows: " + str(hp2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 280
                      
                    # -----------------------------------------------------------------------------------------    
                    # (Step 3/8f) - Windows HANDLE HT2XI
                    # -----------------------------------------------------------------------------------------    
 
                    message2print = "{(Step 3/8f) got" + "; handle-ht2xi-from-CreateProcess-got-by-init-of-Popen4Xi-Windows: " + str(ht2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 256     
                    
                    # -----------------------------------------------------------------------------------------    
                    # (Step 3/8g) - Windows PID2XI
                    # -----------------------------------------------------------------------------------------    
                    message2print = "{(Step 3/8g) got" + "; pid2xi-from-CreateProcess-got-by-init-of-Popen4Xi-Windows: " + str(pid2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 5524
                
                    # -----------------------------------------------------------------------------------------    
                    # (Step 3/8h) - Windows TID2XI
                    # -----------------------------------------------------------------------------------------    
                    message2print = "{(Step 3/8h) got" + "; tid2xi-from-CreateProcess-got-by-init-of-Popen4Xi-Windows: " + str(tid2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)   
                    # 5640
                
                    try:
                        # -----------------------------------------------------------------------------------------    
                        # (Step 3/8i) - Windows CLOSE-OF-HANDLE-HT2XI
                        # -----------------------------------------------------------------------------------------    
                        message2print = "{(Step 3/8i) passing" + "; ht2xi-passing-to-_winapi.CloseHandle-by-init-of-Popen4Xi-Windows: " + str(ht2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)     
                        # 256
                                                 
                        _winapi.CloseHandle(ht2xi)
        
                        message2print = "{(Step 3/8j) performed" + "; CloseHandle-performed-by-init-of-Popen4Xi-Windows: " + str(_winapi.CloseHandle) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                        # <built-in function CloseHandle>
                        
                    except:
                        message2print = "{(Step 3/8z1) failed" + "; CloseHandle-failed-by-init-of-Popen4Xi-Windows: " + str(_winapi.CloseHandle) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                 
                except:
                    message2print = "{(Step 3/8z2) failed" + "; CreateProcess-failed-by-init-of-Popen4Xi-Windows: " + str(_winapi.CreateProcess) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                
                message2print = "{(Step 3/8k) performed" + "; CreateProcess-performed-by-init-of-Popen4Xi-Windows: " + str(_winapi.CreateProcess) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # <built-in function CreateProcess>
                    
                # -------------------------------------------------------------------------------------
                # (Step 4/8) - Windows CLOSE (not using finally)  
                #   Ref:     http://bytes.com/groups/python/854672-os-pipe-_winapi-under-windows
                #            http://mail.python.org/pipermail/python-checkins/2008-July/072006.html
                #            fix issue3120 - don't truncate handles on 64-bit Windows
                # -------------------------------------------------------------------------------------
                message2print = "{(Step 4/8a) closing" + "; read-handle2xi-closing-by-init-of-Popen4Xi-Windows: " + str(rhandle2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 268
                
                try: 
    
                    message2print = "{(Step 4/8b) calling" + "; os-close-calling-by-init-of-Popen4Xi-Windows: " + str(os.close) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)       
                    # <built-in function close>
                    
                    message2print = "{(Step 4/8c) closing" + "; rhandle2xi-from-duplicate-closing-by-init-of-Popen4Xi-Windows: " + str(rhandle2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    # 268
                           
                    close4xi(rhandle2xi) 
                                 
  
                    message2print = "{(Step 4/8d) performed" + "; close4xi-performed-by-init-of-Popen4Xi-Windows: " + str(close4xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    # <function close4xi at 0x00000000033B01E0>                                   
                except: 
                    message2print = "{(Step 4/8e) hit" + "; exception-hit-by-init-of-Popen4Xi-Windows: " + " *** OS error 9: bad file descriptor with os.close: " + str(os.close) + " parameter: rhandle2xi = " + str(rhandle2xi) + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # dragonized solution: *** OS error 9: bad file descriptor with os.close: <built-in function close> parameter: rhandle2xi = 244 *** with os.close(rhandle2xi)
                                        

                    
                # -------------------------------------------------------------------------------------
                # Step (5/8): - Windows SET ATTRIBUTES OF SELF
                # -------------------------------------------------------------------------------------
                message2print = "{(Step 5/8a) using" + "; pid2xi-from-_winapi-using-by-init-of-Popen4Xi-Windows: " + str(pid2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 5524
                
                self.pid = pid2xi

                   
                message2print = "{(Step 5/8b) set" + "; pid-of-popen2xi-set-by-init-of-Popen4Xi-Windows to: " + str(self.pid)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 5524
                                
                self.returncode = None

                message2print = "{(Step 5/8c) set" + "; return-code-of-popen2xi-set-by-init-of-Popen4Xi-Windows to: " + str(self.returncode)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
                # None
                
                message2print = "{(Step 5/8d) using" + "; handle-hp2xi-from-_winapi-using-by-init-of-Popen4Xi-Windows: " + str(hp2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 280
                
                self._handle = hp2xi
 
                message2print = "{(Step 5/8e) set" + "; handle-of-popen2xi-set-by-init-of-Popen4Xi to: " + str(self._handle)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
                # 280 
                
                self.sentinel = int(hp2xi)
                message2print = "{(Step 5/8f) set" + "; sentinel-of-popen2xi-set-by-init-of-Popen4Xi to: " + str(self.sentinel)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
                # 280 
                
                # -------------------------------------------------------------------------------------
                # (Step 6/8a): - Windows SEND INFORMATION TO CHILD
                # -------------------------------------------------------------------------------------
         
                message2print = "{(Step 6/8a) commented" + "; sending-commented-by-init-of-Popen4Xi-Windows: preparing to send information to child " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 10:59:28 01/21/13 Pacific Standard Time
                            
                # -------------------------------------------------------------------------------------
                # Step 6/8b: Windows 
                # -------------------------------------------------------------------------------------              
                message2print = "{(Step 6/8b) passing" + "; process-object-name-passing-to-get-preparation-data4xi-by-init-of-Popen4Xi-Windows: " + str(process_obj._name) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # Process4Xi-1
                
                prep_data2xi = get_preparation_data4xi(process_obj._name) # gets dictionary about parent
                
             
                message2print = "{(Step 6/8b) set" + "; prep-data2xi-set-by-init-of-Popen4Xi to: " + str(prep_data2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # {'name': 'Process4Xi-1', 'authkey': b'\x9at\xc4\xbck0s~t\xbb\xda>\xbf\xfc\x92\x99D\x0f\xb3\xf7_\x88\x91m\xa2D\x90\xee\xf6\xad\x9e\x99', 
                #  'sys_argv': ['hello_world4xi.py'], 'orig_dir': 'E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi', 
                #               'main_path': 'E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi\\hello_world4xi.py', 
                #               'sys_path': ['E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi', 'd:\\Python33', 'd:\\Python33\\python33.zip', 
                #                            'd:\\Python33\\DLLs', 'd:\\Python33\\lib', 'd:\\Python33\\lib\\site-packages', 'd://python33//lib//multiprocessing//', 
                #                            'e://sw4xi//python33//lib//multiprocessing4xi//', 'e://studio4xi//python//3.3//src//',
                #                            'e://scope4xi//041721//zap//lib//c//' ... 'e://scope4xi//041721//zap//lib//_//'],
                #   'log_to_stderr': True
                #  }
                
                # -------------------------------------------------------------------------------------
                # (Step 7/8a) - Windows 
                # -------------------------------------------------------------------------------------
            
                message2print = "{(Step 7/8a1) passing" + "; write-file-descriptor2xi-from-os-pipe-passing-to-os-fdopen-by-init-of-Popen4Xi-Windows: " + str(wfd2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 4
                                
                message2print = "{(Step 7/8a2) passing" + "; wb-passing-to-os-fdopen-by-init-of-Popen4Xi-Windows: " + 'wb' + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # wb
                
                
                # -------------------------------------------------------------------------------------
                # (Step 7/8b): - Windows     
                # -------------------------------------------------------------------------------------            
                
                to_child2xi = os.fdopen(wfd2xi, 'wb')                  # TO BE SOLVED; why is wfd2xi not closed?
    
             
                message2print = "{(Step 7/8b) set" + "; to-child2xi-set-by-init-of-Popen4Xi to: " + str(to_child2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # <_io.BufferedWriter name=4>               

                
                # -------------------------------------------------------------------------------------
                # (Step 7/8c): - Windows- sending information to child          
                # -------------------------------------------------------------------------------------              
                message2print = "{(Step 7/8c1) passing" + "; handle-hp2xi-from-_winapi-passing-to-int-by-init-of-Popen4Xi-Windows: " + str(hp2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # 280
                
                Popen4Xi._tls.process_handle = int(hp2xi)            # thread local storage handle
    
            
                message2print = "{(Step 7/8c2) set" + "; thread-local-storage-process-handle-of-Popen4Xi-set-by-init-of-Popen4Xi to: " + str(Popen4Xi._tls.process_handle)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 280
                
                # -------------------------------------------------------------------------------------
                # (Step 7/8d-f): - Windows           
                # -------------------------------------------------------------------------------------                
  
                message2print = "{(Step 7/8d) passing" + "; prep-data2xi-passing-to-dump-by-init-of-Popen4Xi-Windows: " + str(prep_data2xi) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)            
                # {'name': 'Process4Xi-1', 
                #  'authkey': b'\x9at\xc4\xbck0s~t\xbb\xda>\xbf\xfc\x92\x99D\x0f\xb3\xf7_\x88\x91m\xa2D\x90\xee\xf6\xad\x9e\x99', 
                #  'sys_argv': ['hello_world4xi.py'],
                #  'orig_dir': 'E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi',
                #  'main_path': 'E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi\\hello_world4xi.py',
                #  'sys_path': ['E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi', 'd:\\Python33', 'd:\\Python33\\python33.zip', 'd:\\Python33\\DLLs',
                #               'd:\\Python33\\lib', 'd:\\Python33\\lib\\site-packages', 'd://python33//lib//multiprocessing//', 'e://sw4xi//python33//lib//multiprocessing4xi//',
                #               'e://studio4xi//python//3.3//src//', 'e://sw4xi//python33//zap//', 
                #               'e://scope4xi//041721//zap//lib//a//' ... 'e://scope4xi//041721//zap//lib//_//'],
                #  'log_to_stderr': True
                # }


                message2print = "{(Step 7/8e) passing" + "; to-child2xi-passing-to-dump-by-init-of-Popen4Xi-Windows: " + str(to_child2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <_io.BufferedWriter name=4>
                
                message2print = "{(Step 7/8f) passing" + "; HIGHEST-PROTOCOL-passing-to-dump-by-init-of-Popen4Xi-Windows: " + str(HIGHEST_PROTOCOL) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 3
                
                try: 
    
                    # -------------------------------------------------------------------------------------    
                    # DUMP FIRST THE PREPARED DATA TO CHILD - OK 
                    # ------------------------------------------------------------------------------------- 
                  
                    dump(prep_data2xi, to_child2xi, HIGHEST_PROTOCOL)
                
 
                    message2print = "{(Step 7/8g) dumped" + "; prep-data2xi-dumped-by-init-of-Popen4Xi-Windows: " + str(prep_data2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
                    # {'name': 'Process4Xi-1', 
                    #  'authkey': b'\x9at\xc4\xbck0s~t\xbb\xda>\xbf\xfc\x92\x99D\x0f\xb3\xf7_\x88\x91m\xa2D\x90\xee\xf6\xad\x9e\x99', 
                    #  'sys_argv': ['hello_world4xi.py'],
                    #  'orig_dir': 'E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi',
                    #  'main_path': 'E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi\\hello_world4xi.py',
                    #  'sys_path': ['E:\\SW4XI\\Python33\\Lib\\multiprocessing4xi', 'd:\\Python33', 'd:\\Python33\\python33.zip', 'd:\\Python33\\DLLs',
                    #               'd:\\Python33\\lib', 'd:\\Python33\\lib\\site-packages', 'd://python33//lib//multiprocessing//', 'e://sw4xi//python33//lib//multiprocessing4xi//',
                    #               'e://studio4xi//python//3.3//src//', 'e://sw4xi//python33//zap//', 
                    #               'e://scope4xi//041721//zap//lib//a//' ... 'e://scope4xi//041721//zap//lib//_//'],
                    #  'log_to_stderr': True
                    # }
                    
                    message2print = "{(Step 7/8h1) dumped" + "; to-child2xi-dumped-by-init-of-Popen4Xi-Windows: " + str(to_child2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
                    # <_io.BufferedWriter name=4>
                    
                    message2print = "{(Step 7/8h2) dumped" + "; HIGHEST-PROTOCOL-dumped-by-init-of-Popen4Xi-Windows: " + str(HIGHEST_PROTOCOL) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
                    # 3     
                except:
    
                    # -------------------------------------------------------------------------------------    
                    #
                    # ------------------------------------------------------------------------------------- 
                  
                    message2print = "{(Step 7/8i) hit" + "; exception-hit-by-init-of-Popen4Xi-Windows: " + " *** function: " + str(dump) + " prep_data2xi: " + str(prep_data2xi) + " to_child2xi: " + str(to_child2xi) + "HIGHEST_PROTOCOL: " + str(HIGHEST_PROTOCOL) + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
    
                    
                    
                # -------------------------------------------------------------------------------------
                # (Step 7/8j) : OK - Windows             
                # -------------------------------------------------------------------------------------               
     
                message2print = "{(Step 7/8j) passing" + "; process-object-passing-to-dump-by-init-of-Popen4Xi-Windows: " + str(process_obj) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                # <Process4Xi(Process4Xi-1, initial)>
                                
                message2print = "{(Step 7/8k) passing" + "; to-child2xi-passing-to-dump-by-init-of-Popen4Xi-Windows: " + str(to_child2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # <_io.BufferedWriter name=4>
                                     
                message2print = "{(Step 7/8l) passing" + "; HIGHEST_PROTOCOL-passing-to-dump-by-init-of-Popen4Xi-Windows: " + str(HIGHEST_PROTOCOL) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 3
                
                message2print = "{(Step 7/8m) calling" + "; dump-calling-by-init-of-Popen4Xi-Windows: " + str(dump) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)       
                # <function dump at 0x00000000033B0158>
                try:                    
    
                    # -------------------------------------------------------------------------------------    
                    # DUMP SECOND THE PROCESS OBJECT TO CHILD - OK  - Windows  
                    # ------------------------------------------------------------------------------------- 
             
                    
                    dump(process_obj, to_child2xi, HIGHEST_PROTOCOL) #to be solved the latest 
                
                 
                    message2print = "{(Step 7/8n) dumped" + "; process-object-dumped-by-init-of-Popen4Xi-Windows: " + str(process_obj) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
                    # <Process4Xi(Process4Xi-1, initial)>
                    
                    message2print = "{(Step 7/8o1) dumped" + "; to-child2xi-dumped-by-init-of-Popen4Xi-Windows: " + str(to_child2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # <_io.BufferedWriter name=4>
                    
                    message2print = "{(Step 7/8o2) dumped" + "; HIGHEST-PROTOCOL-dumped-by-init-of-Popen4Xi-Windows: " + str(HIGHEST_PROTOCOL) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                                             
                    # 3
                except:
                  
                    message2print = "{(Step 7/8p) hit" + "; exception-hit-by-init-of-Popen4Xi-Windows: " + " *** function: " + str(dump) + " process_obj: " + str(process_obj) + " to_child2xi: " + str(to_child2xi) + " HIGHEST_PROTOCOL: " + str(HIGHEST_PROTOCOL) + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    


                    
                    
                # -------------------------------------------------------------------------------------
                # (Step 7/8q) OK  - Windows JATKA          
                # -------------------------------------------------------------------------------------                    
                finally: 
                    message2print = "{(Step 7/8q) deleting" + "; Popen4Xi-thread-local-storage-process-handle-deleting-by-init-of-Popen4Xi-Windows: " + str(Popen4Xi._tls.process_handle) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    # 280
                    
                    try:
                        del Popen4Xi._tls.process_handle #delete thread local storage handle
                    
                        # to_child2xi.close() <-- not needed anymore in newer versions
                        # message2print = "{(Step 7/8r) performed" + "; to-child2xi-close-performed-by-init-of-Popen4Xi-Windows: " + str(to_child2xi.close) + "; time: " + time4xi() + "}"
                        # print4xi(class2print, method2print, message2print)             
                        # Windows Vista <built-in method close of _io.BufferedWriter object at 0x02B04730>
                        

                    except:
                        message2print = "{(Step 7/8z) yielded" + "; deleting-yielded-by-init-of-Popen4Xi-Windows: deleting of thread local storage handle yielded ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # Windows 7 10:59:28 01/21/13 Pacific Standard Time  <-- with to_child2xi.close()  
                    
                # -------------------------------------------------------------------------------------
                # Step 7/8 challenges met - Windows 
                # -------------------------------------------------------------------------------------            
                # 1) ForkingPickler4Xi embeds pickle and MakeProxyType4Xi function is embedded inside each module using it
                #       solved:      pickle.py: klass = getattr(mod, name): module object has no attribute PoolProxy4Xi 
                #                    can't pickle makeproxytype4xi.PoolProxy4Xi: it's not found as makeproxytype.PoolProxy4Xi     
                # 2) SyncManager4Xi.register('Pool4Xi', Pool4Xi, PoolProxy4Xi) <-- syncmanager4xi.py
                # 3) pool4xi.py and pool.py 
                # 4) why does the old forking.py Class Popen #line 343 get called !!!! 
                
                # -------------------------------------------------------------------------------------
                # (Step 8/8) - Windows 
                # -------------------------------------------------------------------------------------
                message2print = "{(Step 8/8a) activated" + "; popen2xi-of-Popen4Xi-Windows-activated: " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 10:59:28 01/21/13 Pacific Standard Time
                           
                message2print = "{(Step 8/8b) inherited" + "; object-inherited-by-popen2xi-of-Popen4Xi-Windows: " + str(object)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
                # <class 'object'>
                    
                message2print = "{(Step 8/8c) shared" + "; thread-local-storage-shared-by-popen2xi-of-Popen4Xi-Windows: " + str(Popen4Xi._tls)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                      
                # <_thread._local object at 0x000000000343AEB8>
                            
                message2print = "{(Step 8/8d) returning" + "; init-of-Popen4Xi-Windows-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                # 10:59:28 01/21/13 Pacific Standard Time
                
        #</methodize4xi>
     
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "thread-is-spawning-of-Popen4Xi" usage = "returns whether thread local storage process handle is none or not"> 
        #<note4xi note = "Windows OS"/>
        #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
        # -----------------------------------------------------------------------------------------
        @staticmethod 
        def thread_is_spawning(): 
            '''
            returns whether thread local storage process handle is none or not
            '''            
     
            # -------------------------------------------------------------------------------------    
            # (Step 1/2) - Windows   
            # -------------------------------------------------------------------------------------         
            class2print = "Popen4Xi"
            method2print = "thread_is_spawning"
            
            message2print = "{(Step 1/2a) started" + "; thread-is-spawning-of-Popen4Xi-Windows-started: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)   
            # 
                                       
            message2print = "{(Step 1/2b) using" + "; process-object-using-by-thread-is-spawning-of-Popen4Xi-Windows: " + str(process_obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #                 
                      
            # -------------------------------------------------------------------------------------    
            # - Windows 
            # -------------------------------------------------------------------------------------                 
            if examine4xi(class2print, method2print):
        
                # ---------------------------------------------------------------------------------    
                # StepOriginal
                # ---------------------------------------------------------------------------------             
                message2print = "{(StepOriginal) returning" + "; Popen-thread-is-spawning-function-returning-by-thread-is-spawning-of-Popen4Xi-Windows: " + str(Popen.thread_is_spawning) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
                             
                return Popen.thread_is_spawning()
            else:            
                # -------------------------------------------------------------------------------------
                # (Step 2/2) - Windows           
                # -------------------------------------------------------------------------------------              
                message2print = "{(Step 2/2a) passing" + "; Popen4Xi-tls-passing-to-getattr-by-thread-is-spawning-of-Popen4Xi-Windows: " + str(Popen4Xi._tls) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 2/2b) passing" + "; process-handle-passing-to-getattr-by-thread-is-spawning-of-Popen4Xi-Windows: " + str('process_handle') + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
                
                message2print = "{(Step 2/2c) returning" + "; getattr-function-returning-by-thread-is-spawning-of-Popen4Xi-Windows: " + str(getattr) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
                                             
                return getattr(Popen4Xi._tls, 'process_handle', None) is not None  #thread local storage handle is passed
        #</methodize4xi>
            
            
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "duplicate-of-child-of-Popen4Xi" usage = "returns duplicate">
        #<note4xi note = "Windows OS"/>
        #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
        # -----------------------------------------------------------------------------------------
        @staticmethod
        def duplicate_for_child(handle):
            '''
            returns duplicate
            '''
            
      
            # -------------------------------------------------------------------------------------    
            # (Step 1/2) - Windows   
            # -------------------------------------------------------------------------------------                 
        
            class2print = "Popen4Xi"
            method2print = "duplicate_for_child"
            
               
            message2print = "{(Step 1/2a) started" + "; duplicate-of-child-of-Popen4Xi-Windows-started: " + str(duplicate_for_child)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            
                                       
            message2print = "{(Step 1/2b) passed" + "; handle-passed-to-duplicate-of-child-of-Popen4Xi-Windows: " + str(handle) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            

            # -------------------------------------------------------------------------------------    
            # - Windows 
            # -------------------------------------------------------------------------------------                 
            if examine4xi(class2print, method2print):
        
                # ---------------------------------------------------------------------------------    
                # StepOriginal
                # ---------------------------------------------------------------------------------
                message2print = "{(StepOriginal) passing" + "; handle-passing-to-forking-duplicate-by-duplicate-of-child-of-Popen4Xi-Windows: " + str(handle) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                
                message2print = "{(StepOriginal) passing" + "; Popen-tls-process-handle-passing-to-forking-duplicate-by-duplicate-of-child-of-Popen4Xi-Windows: " + str(Popen._tls.process_handle) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                               
                
                message2print = "{(StepOriginal) returning" + "; forking-duplicate-function-returning-by-duplicate-of-child-of-Popen4Xi-Windows: " + str(forking.duplicate) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
                                         
                return forking.duplicate(handle, Popen._tls.process_handle)
            
            else:                
                # -------------------------------------------------------------------------------------
                # (Step 2/2) - Windows           
                # -------------------------------------------------------------------------------------         
                message2print = "{(Step 2/2a) passing" + "; handle-passing-to-duplicate-by-duplicate-of-child-of-Popen4Xi-Windows: " + str(handle) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)           
                
                message2print = "{(Step 2/2b) passing" + "; Popen4Xi-tls-process-handle-passing-to-duplicate-by-duplicate-of-child-of-Popen4Xi-Windows: " + str(Popen4Xi._tls.process_handle) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)              
                
                message2print = "{(Step 2/2c) returning" + "; duplicate-function-returning-by-duplicate-of-child-of-Popen4Xi-Windows: " + str(duplicate) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
                   
                return duplicate(handle, Popen4Xi._tls.process_handle) #thread local storage process handle is passed
        #</methodize4xi>
            
    
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "wait-of-Popen4Xi" usage = "waits according to the timeout given">
        #<note4xi note = "Windows OS; make sure NOT to give infinite but instead a limited timeout"/>
        #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
        # -----------------------------------------------------------------------------------------
        def wait(self, timeout=None):
            '''
            waits according to the timeout given             
            '''
      
            # -------------------------------------------------------------------------------------    
            # (Step 1/7) - Windows   
            # -------------------------------------------------------------------------------------                 
            class2print = "Popen4Xi"
            method2print = "wait"
            
            message2print = "{(Step 1/7a) started" + "; wait-of-Popen4Xi-Windows-started: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)   
            # 01:51:06 01/22/13 Pacific Standard Time
             
                                
            # -------------------------------------------------------------------------------------    
            # dragonized by setting timeout and thereby avoiding system to hang up   
            # -------------------------------------------------------------------------------------             
            timeout = omega2timeout4xi(class2print, method2print) 

            message2print = "{(Step 1/7b) set" + "; timeout-set-by-wait-of-Popen4Xi-Windows to: " + str(timeout)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
            # 0.4  <- 2x in new version 
                  
            # -------------------------------------------------------------------------------------    
            # - Windows 
            # -------------------------------------------------------------------------------------                 
            if examine4xi(class2print, method2print):
        
                # ---------------------------------------------------------------------------------    
                # StepOriginal OK
                # ---------------------------------------------------------------------------------             
     
                
                message2print = "{(StepOriginal) passing" + "; timeout-passing-to-Popen-wait-by-wait-of-Popen4Xi-Windows: " + str(timeout) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(StepOriginal) returning" + "; Popen-wait-function-returning-by-wait-of-Popen4Xi-Windows: " + str(Popen.wait) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
                                                
                
                return Popen.wait(self, timeout)
            else:
               
                # -------------------------------------------------------------------------------------
                # (Step 2/7) 
                #       ref: http://pydoc.org/get.cgi/usr/local/lib/python2.4/_winapi.py
                #            http://subdev.blogspot.com/
                #            _winapi.py in the Python 3.1 Lib
                #            http://www.thescarms.com/vbasic/wait.aspx - always give some timeout
                # -------------------------------------------------------------------------------------  
                if self.returncode is None:
                    if timeout is None:
                        msecs = _winapi.INFINITE
                        
                        message2print = "{(Step 2/7a) set" + "; msecs-set-by-wait-of-Popen4Xi to: " + str(msecs)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                    else:
                        if timeout == 0:
                            timeout = 1.0
                            
                            message2print = "{(Step 2/7b) set" + "; timeout-set-by-wait-of-Popen4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            # 
                            
                        msecs = max(0, int(timeout * 800 + 0.5)) # <-- dragonized max(0, int(timeout * 1000 + 0.5)), where msecs = 0 if timeout = 0 ! 
                                                                 #     tested values with new algoritm: 0.2sec => 160ms; 0.4sec => 320ms; 0.8sec=>650ms; 1.0sec => 800ms
                        
                        message2print = "{(Step 2/7c) set" + "; msecs-set-by-wait-of-Popen4Xi to: " + str(msecs)  + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print) 
                        # 800<--old version; 320<-- new version
                                                
                    message2print = "{(Step 2/7d) do" + "; calculating-msecs-do-by-wait-of-Popen4Xi-Windows: " + str(msecs) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                        
                    # 800<--old version; 320<-- new version 
                    
    
                    # -------------------------------------------------------------------------------------    
                    # (Step 3/7) - Windows 
                    # -------------------------------------------------------------------------------------                 
                    message2print = "{(Step 3/7a) passing" + "; handle-of-popen2xi-passing-to-_winapi-WaitForSingleObject-by-wait-of-Popen4Xi-Windows: " + str(self._handle) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    # 280
                    
                    ack2response = False
                    nack2response = True
                    count2try = 0

                    max2count = limit2loop4xi(class2print, method2print) 
                    
                    message2print = "{(Step 3/7b) got" + "; max2count-got-by-wait-of-Popen4Xi-Windows: " + str(max2count)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 8
                    
                    while ((count2try < max2count) and (nack2response is True)):
                    
                        message2print = "{(Step 3/7c) passing" + "; milliseconds-passing-to-_winapi-WaitForSingleObject-by-wait-of-Popen4Xi-Windows: " + str(msecs) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                                        
                        # 320
                        
                        message2print = "{(Step 3/7d) calling" + "; _winapi-WaitForSingleObject-calling-by-wait-of-Popen4Xi-Windows: " + str(_winapi.WaitForSingleObject) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)       
                        # <built-in function WaitForSingleObject>                
             
                        try: 
                            response2xi = _winapi.WaitForSingleObject(int(self._handle), msecs)
                        
                            if response2xi == 0:
                                count2try = count2try + 1
                                
                                
                                message2print = "{(Step 3/7e) yielded" + "; response-yielded-by-wait-of-Popen4Xi-Windows: " + str(response2xi)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                # 0 received as the loop allowed i.e. response is received but it is zero - asyncronous services had challnges in old version - see PEPS ! 
                                # 0 received by new version too!
                                
                                msecs = msecs + 1000
                                
                                message2print = "{(Step 3/7f) upgraded" + "; milliseconds-upgraded-by-wait-of-Popen4Xi-Windows: " + str(msecs)  + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)                                 
                                # 1320 ... 8320
                                
                                omega2sleep4xi(class2print ,method2print)  # dragonized giving time to others 
                                
                            else:
                                nack2response = False
                                ack2response = True 
                                
                                message2print = "{(Step 3/7g) acknowledged" + "; ack2response-acknowledged-by-wait-of-Popen4Xi-Windows: " + str(ack2response) + "; time: " + time4xi() + "}"
                                print4xi(class2print, method2print, message2print)
                                                
                                
                        except:
                            count2try = count2try + 1
                            
                                                            
                            message2print = "{(Step 3/7h) yielded" + "; nack2response-yielded-by-wait-of-Popen4Xi-Windows: " + str(nack2response)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 

                                 
                    if count2try >= max2count:
                            message2print = "{(Step 3/7i) warned" + "; count2try-warned-by-wait-of-Popen4Xi-Windows: " + " ** " + str(count2try)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # ** 8 ** ; time: 01:53:42 01/22/13 Pacific Standard Time
                            
                                       
                        
                        
                        
    
                    # -------------------------------------------------------------------------------------    
                    # (Step 4/7) - Windows JATKA
                    # -------------------------------------------------------------------------------------                 
                      
                    message2print = "{(Step 4/7a) got" + "; response2xi-got-by-wait-of-Popen4Xi-Windows: " + str(response2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 258 <-- older version, 0 <-- new version
                    
                                      
                    if response2xi == _winapi.WAIT_OBJECT_0:
                        answer2xi = _winapi.GetExitCodeProcess(self._handle)
                        
                        message2print = "{(Step 4/7b) got" + "; answer2xi-got-by-wait-of-Popen4Xi-Windows: " + str(answer2xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                
                        
                        if answer2xi == Popen4Xi.cf2xi.get_terminate(): #dragonized TERMINATE4XI
                            
                            # ----------------------------------------------------------------------------    
                            # (Step 5/7) - Windows 
                            # ----------------------------------------------------------------------------                 
                          
                            answer2xi = -signal.SIGTERM
                            
                            message2print = "{(Step 5/7) set" + "; answer2xi-set-by-wait-of-Popen4Xi to: " + str(answer2xi)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                                            
                                            
                            # ----------------------------------------------------------------------------    
                            #
                            # ----------------------------------------------------------------------------                 
                              
                        # --------------------------------------------------------------------------------    
                        # (Step 6/7) - Windows 
                        # -------------------------------------------------------------------------------- 
                               
                        self.returncode = answer2xi
                        
                        message2print = "{(Step 6/7) set" + "; return-code-of-popen2xi-set-by-wait-of-Popen4Xi to: " + str(self.returncode)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                        # 0   
                           

                                                   
                # -------------------------------------------------------------------------------------
                # (Step 7/7) - Windows             
                # -------------------------------------------------------------------------------------  
                message2print = "{(Step 7/7) returning" + "; return-code-of-popen2xi-returning-by-wait-of-Popen4Xi-Windows: " + str(self.returncode) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                # 0        
                                            
                return self.returncode
        
        #</methodize4xi>
                
            
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "poll-of-Popen4Xi" usage = "polls"> 
        #<note4xi note = "Windows OS"/>
        #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
        # -----------------------------------------------------------------------------------------
        def poll(self):
            '''
            polls 
            '''
            
     
            # -------------------------------------------------------------------------------------    
            # (Step 1/2) - Windows   
            # -------------------------------------------------------------------------------------                 
            class2print = "Popen4Xi"
            method2print = "poll"
               
            message2print = "{(Step 1/2) started" + "; poll-of-Popen4Xi-Windows-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 10:59:28 01/21/13 Pacific Standard Time
            
                        
            # -------------------------------------------------------------------------------------
            # (Step 2/2) - Windows             
            # -------------------------------------------------------------------------------------          
                           
            timeout2xi = omega2timeout4xi(class2print, method2print) 

            message2print = "{(Step 2/2a) set" + "; timeout2xi-set-by-poll-of-Popen4Xi-Windows to: " + str(timeout2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            # 0.2
            
            message2print = "{(Step 2/2b) passing" + "; timeout2xi-passing-to-wait-of-popen2xi-by-poll-of-Popen4Xi-Windows: " + str(timeout2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)     
            # 0.2
            
            message2print = "{(Step 2/2b) returning" + "; wait-function-of-popen2xi-returning-by-poll-of-Popen4Xi-Windows: " + str(self.wait) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
            # <bound method Popen4Xi.wait of <popen4xi.Popen4Xi object at 0x000000000337CE48>> 
                                    
            return self.wait(timeout=timeout2xi)
        #</methodize4xi>
            
            
        # -----------------------------------------------------------------------------------------
        #<methodize4xi method = "terminate-of-Popen4Xi" usage = "terminates"> 
        #<note4xi note = "Windows OS"/>
        #<state4xi status = "ACK4tagging; ACK4moduletests; NACK4systemtest"/>
        # -----------------------------------------------------------------------------------------
        def terminate(self):
            '''
            terminates
            '''
      
            class2print = "Popen4Xi"
            method2print = "terminate" 
                       
            # -------------------------------------------------------------------------------------    
            # (Step 1/3) - Windows   
            # -------------------------------------------------------------------------------------                 

            message2print = "{(Step 1/3a) started" + "; terminate-of-Popen4Xi-Windows-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 01:53:43 01/22/13 Pacific Standard Time
            
                           
            # -------------------------------------------------------------------------------------    
            # - Windows 
            # -------------------------------------------------------------------------------------                 
            if examine4xi(class2print, method2print):
        
                # ---------------------------------------------------------------------------------    
                # (StepOriginal): OK - Windows 
                # ---------------------------------------------------------------------------------             
                message2print = "{(StepOriginal) calling" + "; Popen-terminate-calling-by-terminate-of-Popen4Xi-Windows: " + str(Popen.terminate) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                                
                
                Popen.terminate(self) 
                # { Popen4Xi : terminate ; StepOriginal Popen.terminate() time: 17:10:14 09/02/09 FLE Daylight Time}
            else:
                                           
                
                            
                # -------------------------------------------------------------------------------------
                # (Step 2/3) OK4HW  - Windows           
                # -------------------------------------------------------------------------------------                  
                #message2print = "{(Step 2/3a1) shared" + "; cf2xi-of-Popen4Xi-shared-by-terminate-of-Popen4Xi-Windows: " + str(Popen4Xi.cf2xi)  + "; time: " + time4xi() + "}"
                #print4xi(class2print, method2print, message2print)                      

     
                message2print = "{(Step 2/3a2) checking" + "; return-code-of-popen2xi-checking-by-terminate-of-Popen4Xi-Windows: " + str(self.returncode) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)           
                # 0          
                
                if self.returncode is None:
                    message2print = "{(Step 2/3b) passing" + "; handle-of-popen2xi-passing-to-_winapi-TerminateProcess-by-terminate-of-Popen4Xi-Windows: " + str(self._handle) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    #HW: 
                                    
                    message2print = "{(Step 2/3c) passing" + "; terminate-passing-to-_winapi-TerminateProcess-by-terminate-of-Popen4Xi-Windows: " + str(Popen4Xi.cf2xi.get_terminate()) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                                       
                    #HW: { Popen4Xi : terminate ; (Step 2/3c) passing; terminate-passing-to-_winapi-TerminateProcess-by-terminate-of-Popen4Xi-Windows: 65536; time: 20:31:30 01/13/10 FLE Standard Time}
                    
                    try:
                        _winapi.TerminateProcess(int(self._handle),Popen4Xi.cf2xi.get_terminate()) #dragonized TERMINATE4XI
                        
                        message2print = "{(Step 2/3d) performed" + "; _winapi-TerminateProcess-performed-by-terminate-of-Popen4Xi-Windows: " + str(_winapi.TerminateProcess) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                        #HW: { Popen4Xi : terminate ; (Step 2/3d) performed; _winapi-TerminateProcess-performed-by-terminate-of-Popen4Xi-Windows: <built-in function TerminateProcess>; time: 20:31:30 01/13/10 FLE Standard Time}
                        
                                            
                    except WindowsError:
                        message2print = "{(Step 2/3e) hit" + "; WindowsError-hit-by-terminate-of-Popen4Xi-Windows: " + " *** _winapi.TerminateProcess(int(self._handle),Popen4Xi.cf2xi.get_terminate()) *** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                    
                        timeout = omega2timeout4xi(class2print, method2print) 

                        message2print = "{(Step 2/3f) set" + "; timeout-set-by-terminate-of-Popen4Xi-Windows to: " + str(timeout)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                      
                    
                        if self.wait(timeout) is None:
                            message2print = "{(Step 2/3g) hit" + "; wait-of-popen2xi-error-hit-by-terminate-of-Popen4Xi-Windows: " + " *** timeout = " + str(timeout) + " *** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)
                            
                            raise
                # -------------------------------------------------------------------------------------
                # (Step 3/3) - Windows            
                # ------------------------------------------------------------------------------------- 

                
                message2print = "{(Step 3/3) returning" + "; terminate-of-Popen4Xi-Windows-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 
                # 01:53:43 01/22/13 Pacific Standard Time
                
                                         
    #</methodize4xi>
                                           
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "sense-of-Popen4Xi" usage = "senses Popen4Xi class objects"> 
        #<note4xi note = "Windows OS"/>
        #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
        # ---------------------------------------------------------------------------------------------
        def sense(self):
            '''
            method sense: senses Popen4Xi class objects
            '''
                       
            # -------------------------------------------------------------------------------------    
            # (Step 1/5) - Windows   
            # -------------------------------------------------------------------------------------                 
            
            
            class2print = 'Popen4Xi'
            method2print = 'sense'
            
               
            message2print = "{(Step 1/5) started" + "; sense-of-Popen4Xi-Windows-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
  
            
            # -----------------------------------------------------------------------------------------
            # (Step 2/5) - Windows 
            # -----------------------------------------------------------------------------------------  
            message2print = "{(Step 2/5) sensing" + "; Popen4Xi-sensing-by-sense-of-Popen4Xi-Windows: " + str(Popen4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                       
            
            # -----------------------------------------------------------------------------------------
            #  (Step 3/5) get sensed knowledge for this object - Windows 
            # -----------------------------------------------------------------------------------------        
            kb2xi = sense_object4xi(self)
            
            message2print = "{(Step 3/5) got" + "; knowledge-base2xi-got-by-sense-of-Popen4Xi-Windows: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            # -----------------------------------------------------------------------------------------
            #  (Step 4/5) add knowledge specific to this object - Windows 
            # format example: kb2xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------         
            try:
                kb2xi['thread-local-storage-of-popen2xi'] = Popen4Xi._tls
                kb2xi['pid-of-popen2xi'] = self.pid
            except:
                message2print = "{yielded" + "; knowledge-base2xi-yielded-by-sense-of-Popen4Xi-Windows: " + str(kb2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
            
          
            message2print = "{(Step 4/5) sensed" + "; Popen4Xi-sensed-by-sense-of-Popen4Xi-Windows: " + str(Popen4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/5) - Windows 
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 5/5) returning" + "; knowledge-base2xi-returning-by-sense-of-Popen4Xi-Windows: " + str(kb2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                               
              
            return kb2xi
            
            # -----------------------------------------------------------------------------------------
            #
            # -----------------------------------------------------------------------------------------        
        #</methodize4xi>
        
            
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "reason-of-Popen4Xi" usage = "reasons Popen4Xi class objects"> 
        #<note4xi note = "Windows OS"/>
        #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
        # ---------------------------------------------------------------------------------------------
        def reason(self):
            '''
            method reason: reasons Popen4Xi class objects
            '''
                       
            # -------------------------------------------------------------------------------------    
            # (Step 1/5) - Windows   
            # -------------------------------------------------------------------------------------                 
                
            
            class2print = 'Popen4Xi'
            method2print = 'reason'
    
               
            message2print = "{(Step 1/5) started" + "; reason-of-Popen4Xi-Windows-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
           
            
            # -----------------------------------------------------------------------------------------
            # (Step 2/5) - Windows 
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 2/5) reasoning" + "; Popen4Xi-reasoning-by-reason-of-Popen4Xi-Windows: " + str(Popen4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/5) get reasoned knowledge for this object - Windows 
            # -----------------------------------------------------------------------------------------         
            kb4xi = reason_object4xi(self)
            
            message2print = "{(Step 3/5) got" + "; knowledge-base4xi-got-by-reason-of-Popen4Xi-Windows: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            # -----------------------------------------------------------------------------------------
            # (Step 4/5) add knowledge specific to this object - Windows 
            # format example: kb4xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------          
            
            
              
            
            message2print = "{(Step 4/5) reasoned" + "; Popen4Xi-reasoned-by-sense-of-Popen4Xi-Windows: " + str(Popen4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/5) - Windows 
            # -----------------------------------------------------------------------------------------        
            message2print = "{(Step 5/5) returning" + "; knowledge-base4xi-returning-by-reason-of-Popen4Xi-Windows: " + str(kb4xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                        
              
            return kb4xi
            
        #</methodize4xi>
            
    
        
        # ---------------------------------------------------------------------------------------------
        #<methodize4xi method = "ace-of-Popen4Xi" usage = "aces Popen4Xi class objects"> 
        #<note4xi note = "Windows OS"/>
        #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
        # ---------------------------------------------------------------------------------------------
        def ace(self):
            '''
            method ace: aces Popen4Xi class objects 
            '''
                    
                       
            # -------------------------------------------------------------------------------------    
            # (Step 1/5) - Windows   
            # -------------------------------------------------------------------------------------                 
            
            
            class2print = 'Popen4Xi'
            method2print = 'ace'
            
               
            message2print = "{(Step 1/5) started" + "; ace-of-Popen4Xi-Windows-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
          
            
            # -----------------------------------------------------------------------------------------
            # (Step 2/5) - Windows 
            # -----------------------------------------------------------------------------------------
            message2print = "{(Step 2/5) acing" + "; Popen4Xi-acing-by-ace-of-Popen4Xi-Windows: " + str(Popen4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                      
            
            # -----------------------------------------------------------------------------------------
            #  (Step 3/5) get aced knowledge for this object - Windows 
            # -----------------------------------------------------------------------------------------        
            kb8xi = ace_object4xi(self)  
            
            message2print = "{(Step 3/5) got" + "; knowledge-base8xi-got-by-ace-of-Popen4Xi-Windows: " + str(kb8xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
    
    
            # -----------------------------------------------------------------------------------------
            # (Step 4/5) add knowledge specific to this object - Windows 
            # format example: kb8xi['time2xi'] = time4xi() #adding time 
            # -----------------------------------------------------------------------------------------              
            
              
            message2print = "{(Step 4/5) aced" + "; Popen4Xi-aced-by-ace-of-Popen4Xi-Windows: " + str(Popen4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/5) - Windows  
            # -----------------------------------------------------------------------------------------        
            message2print = "{(Step 5/5) returning" + "; knowledge-base8xi-returning-by-ace-of-Popen4Xi-Windows: " + str(kb8xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                              
            
            return kb8xi
            
            # -----------------------------------------------------------------------------------------
            # - Windows 
            # ----------------------------------------------------------------------------------------- 
    #</methodize4xi>
    #</class4xi>

    
    # ---------------------------------------------------------------------------------------------
    #<function4xi function = "is-forking-of-Popen4Xi" usage = "
    #    freeze_support():
    #        if is_forking(sys.argv):
    #    get_command_line():
    #        if __name__ == '__main__':
    #            freeze_support()
    #"> 
    #<note4xi note = "Windows OS"/>
    #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
    # ---------------------------------------------------------------------------------------------
    def is_forking(argv):
        '''
        Return whether commandline indicates we are forking
        '''
      
                       
        # -------------------------------------------------------------------------------------    
        # (Step 1/3) - Windows   
        # -------------------------------------------------------------------------------------                 

        class2print = "is_forking"
        method2print = "is_forking"
               
        message2print = "{(Step 1/3a) started" + "; is-forking-of-Popen4Xi-Windows-started: " + str(is_forking)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HW: #PX: <function is_forking at 0x02EAFD68> 
                                       
        message2print = "{(Step 1/3b) passed" + "; argv-passed-to-is-forking-of-Popen4Xi-Windows: " + str(argv) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: ['hello_world4xi.py']
        #PX: ['test_proxy_mp_xi4.py']
           

        # -------------------------------------------------------------------------------------    
        # - Windows 
        # -------------------------------------------------------------------------------------                 
      
        if examine4xi(class2print, method2print):
        
            # ---------------------------------------------------------------------------------    
            # StepOriginal - Windows 
            # ---------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) passing" + "; argv-passing-to-Popen-is-forking-by-is-forking-of-Popen4Xi-Windows: " + str(argv) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
            message2print = "{(StepOriginal) returning" + "; Popen-is-forking-function-returning-by-is-forking-of-Popen4Xi-Windows: " + str(Popen.is_forking) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                                        
            return Popen.is_forking(argv)
        else:
                
            # -----------------------------------------------------------------------------------------
            # (Step 2/3) - Windows            
            # -----------------------------------------------------------------------------------------          
            message2print = "{(Step 2/3a) started" + "; Step-2-of-3-of-is-forking-of-Popen4Xi-Windows-started: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            #HW, #PX: 13:21:25 01/14/10 FLE Standard Time                  
            
            message2print = "{(Step 2/3b) got" + "; lenght-argv-got-by-is-forking-of-Popen4Xi-Windows: " + str(len(argv))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #HW, #PX: 1        

            message2print = "{(Step 2/3c) checking" + "; commandline-checking-by-is-forking-of-Popen4Xi-Windows: " + "checking whether commandline indicates we are forking " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                            
            
            if len(argv) >= 2 and argv[1] == '--multiprocessing-fork':
                assert len(argv) == 3
                
                message2print = "{(Step 2/3b) asserted" + "; lenght-argv--asserted-by-is-forking-of-Popen4Xi-Windows: " + str(len(argv)) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                   
                
                message2print = "{(Step 2/3c) returning" + "; true-returning-by-is-forking-of-Popen4Xi-Windows: " + str(True) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                                        
                return True
            else:
            
                message2print = "{(Step 2/3d) returning" + "; false-returning-by-is-forking-of-Popen4Xi-Windows: " + str(False) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                #HW: False                  
                        
                return False
        # -----------------------------------------------------------------------------------------
        # (Step 3/3) - Windows            
        # ----------------------------------------------------------------------------------------- 
        
    #</function4xi>
     
    # ---------------------------------------------------------------------------------------------
    #<function4xi function = "freeze-support-of-Popen4Xi" usage = "
    # Run code for process object if this in not the main process
    #"> 
    #<note4xi note = "Windows OS"/>
    #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
    # ---------------------------------------------------------------------------------------------
    def freeze_support():
        '''
        Run code for process object if this in not the main process
        '''
        
                       
        # -------------------------------------------------------------------------------------    
        # (Step 1/4) - Windows   
        # -------------------------------------------------------------------------------------                 
                
        class2print = "freeze_support"
        method2print = "freeze_support"
               
        message2print = "{(Step 1/4a) started" + "; freeze-support-of-Popen4Xi-Windows-started: " + str(freeze_support)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/4) - Windows           
        # -----------------------------------------------------------------------------------------         
       
        message2print = "{(Step 2/4) commented" + "; freeze-support-commented-by-freeze-support-of-Popen4Xi-Windows: Run code for process object if this in not the main process " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        # -------------------------------------------------------------------------------------    
        #
        # -------------------------------------------------------------------------------------                 
        if examine4xi(class2print, method2print):
        
            # ---------------------------------------------------------------------------------    
            # StepOriginal - Windows 
            # ---------------------------------------------------------------------------------             

            message2print = "{(StepOriginal) calling" + "; Popen-freeze-support-calling-by-freeze-support-of-Popen4Xi-Windows: " + str(Popen.freeze_support) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                            
            Popen.freeze_support()
        else:
                
            # -----------------------------------------------------------------------------------------
            # (Step 3/4) - Windows           
            # -----------------------------------------------------------------------------------------      
            if is_forking(sys.argv):
                
                message2print = "{(Step 3/4a) calling" + "; main-calling-by-freeze-support-of-Popen4Xi-Windows: " + str(main) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
        
                main()
                
                message2print = "{(Step 3/4b) performed" + "; main-performed-by-freeze-support-of-Popen4Xi-Windows: " + str(main) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                          
                message2print = "{(Step 3/4c) calling" + "; sys-exit-calling-by-freeze-support-of-Popen4Xi-Windows: " + str(sys.exit) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                                
                                 
                sys.exit()
            # -----------------------------------------------------------------------------------------
            # (Step 4/4) - Windows           
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(Step 4/4) returning" + "; freeze-support-of-Popen4Xi-Windows-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
                            
                                                
    #</function4xi>
         
    # ---------------------------------------------------------------------------------------------
    #<function4xi function = "get-command-line-of-Popen4Xi" usage = "
    #    Popen(object):
    #        __init__:
    #            cmd = get_command_line() + [rhandle]
    #"> 
    #<note4xi note = "Windows OS"/>
    #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
    # ---------------------------------------------------------------------------------------------
    def get_command_line():
        '''
        Returns prefix of command line used for spawning a child process
        '''
                       
        # -----------------------------------------------------------------------------------------    
        # (Step 0/3) - Windows   
        # -----------------------------------------------------------------------------------------                 

        class2print = "get_command_line"
        method2print = "get_command_line"
        
               
        message2print = "{(Step 0/3a) started" + "; get-command-line-of-Popen4Xi-Windows-started: " + str(get_command_line)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #HW: <function get_command_line at 0x02D190C0>
          
                   
        # -------------------------------------------------------------------------------------    
        # - Windows 
        # -------------------------------------------------------------------------------------                 
        if examine4xi(class2print, method2print):
        
            # ---------------------------------------------------------------------------------    
            # StepOriginal
            # ---------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) returning" + "; Popen-get-command-line-function-returning-by-get-command-line-of-Popen4Xi-Windows: " + str(Popen.get_command_line) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                
            
            return Popen.get_command_line()               
        else:
         
            # -----------------------------------------------------------------------------------------
            # (Step 1/3) - Windows            
            # -----------------------------------------------------------------------------------------        
            identity2xi = get_identity_current_process4xi()                                 #dragonized
            

            message2print = "{(Step 1/3) got" + "; identity2xi-got-by-get-command-line-of-Popen4Xi-Windows: " + str(identity2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #HW: { get_command_line : get_command_line ; (Step 1/3) got; identity2xi-got-by-get-command-line-of-Popen4Xi-Windows: (); time: 14:18:29 01/14/10 FLE Standard Time}                
            
            # -----------------------------------------------------------------------------------------
            # (Step 2/3) - Windows            
            # -----------------------------------------------------------------------------------------                 
            if identity2xi==() and is_forking(sys.argv):    #dragonized <--- to be done
            
                message2print = "{(Step 2/3) hit" + "; RuntimeError-hit-by-get-command-line-of-Popen4Xi-Windows: " + " *** Attempt to start a new process before the current process has finished its bootstrapping phase *** " + "; time: " + time4xi() + "}"
                
                print4xi(class2print, method2print, message2print)
                    
                     
            
                raise RuntimeError('''
                Attempt to start a new process before the current process
                has finished its bootstrapping phase.
    
                This probably means that you are on Windows and you have
                forgotten to use the proper idiom in the main module:
    
                    if __name__ == '__main__':
                        freeze_support()
                        ...
    
                The "freeze_support()" line can be omitted if the program
                is not going to be frozen to produce a Windows executable.
                ''')
            
            

                            
            # -----------------------------------------------------------------------------------------
            # (Step (3/3) - Windows            
            # ----------------------------------------------------------------------------------------- 
             
             
            if getattr(sys, 'frozen', False):
                message2print = "{(Step (3/3a) returning" + "; sys-executable-returning-by-get-command-line-of-Popen4Xi-Windows: " + str(sys.executable) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                
                               
                message2print = "{(Step (3/3b) returning" + "; multiprocessing-fork-returning-by-get-command-line-of-Popen4Xi-Windows: " + str('--multiprocessing-fork') + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)                  
                               
            
                return [sys.executable, '--multiprocessing-fork']
            else:
                prog = 'from popen4xi import main'                            
                                                                              
                
                message2print = "{(Step (3/3c) set" + "; prog-set-by-get-command-line-of-Popen4Xi to: " + str(prog)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #HW: { get_command_line : get_command_line ; (Step (3/3c) set; prog-set-by-get-command-line-of-Popen4Xi to: from popen4xi import main; time: 14:18:29 01/14/10 FLE Standard Time}
                                
                message2print = "{(Step (3/3d) returning" + "; python-exe-returning-by-get-command-line-of-Popen4Xi-Windows: " + str(_python_exe) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                #HW: { get_command_line : get_command_line ; (Step (3/3d) returning; python-exe-returning-by-get-command-line-of-Popen4Xi-Windows: c:\sw\python32\python.exe; time: 14:18:29 01/14/10 FLE Standard Time}
                
                                
                message2print = "{(Step (3/3e) returning" + "; c-returning-by-get-command-line-of-Popen4Xi-Windows: " + str('-c') + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                #HW: { get_command_line : get_command_line ; (Step (3/3e) returning; c-returning-by-get-command-line-of-Popen4Xi-Windows: -c; time: 14:18:29 01/14/10 FLE Standard Time}
                                
                message2print = "{(Step (3/3f) returning" + "; prog-returning-by-get-command-line-of-Popen4Xi-Windows: " + str(prog) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                #HW: { get_command_line : get_command_line ; (Step (3/3f) returning; prog-returning-by-get-command-line-of-Popen4Xi-Windows: from popen4xi import main; time: 14:18:29 01/14/10 FLE Standard Time}                                                       
                                
                message2print = "{(Step (3/3g) returning" + "; multiprocessing-fork-returning-by-get-command-line-of-Popen4Xi-Windows: " + str('--multiprocessing-fork') + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #HW: { get_command_line : get_command_line ; (Step (3/3g) returning; multiprocessing-fork-returning-by-get-command-line-of-Popen4Xi-Windows: --multiprocessing-fork; time: 14:18:29 01/14/10 FLE Standard Time}
                    
                  
                                
                return [_python_exe, '-c', prog, '--multiprocessing-fork']
        # -----------------------------------------------------------------------------------------
        #           
        # ----------------------------------------------------------------------------------------- 
    #</function4xi>
     
        
    # ---------------------------------------------------------------------------------------------
    #<function4xi function = "main-of-Popen4Xi" usage = "
    #     forking.py main
    #         freeze_support():
    #             if is_forking(sys.argv):
    #                 main()
    #">
    #<note4xi note = "Windows OS"/>
    #<state4xi status = "ACK4tagging; NACK4moduletests; NACK4systemtest"/>
    # ---------------------------------------------------------------------------------------------
    def main():
        '''
        Run code specifed by data received over pipe
        '''
     
                       
        # -----------------------------------------------------------------------------------------    
        # (Step 1/12) 
        # -----------------------------------------------------------------------------------------                 
             
        class2print = "main"
        method2print = "main"
               
        message2print = "{(Step 1/12a) started" + "; main-of-Popen4Xi-started: " + str(main)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
         
                 
        # -------------------------------------------------------------------------------------    
        #
        # -------------------------------------------------------------------------------------                 
        if examine4xi(class2print, method2print):
        
            # ---------------------------------------------------------------------------------    
            # StepOriginal
            # ---------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) calling" + "; Popen-main-calling-by-main-of-Popen4Xi: " + str(Popen.main) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                
            Popen.main()
        else:
                    
            # -----------------------------------------------------------------------------------------
            # (Step 2/12)           
            # -----------------------------------------------------------------------------------------                  
            message2print = "{(Step 2/12a) got" + "; sys-argv-1-got-by-main-of-Popen4Xi: " + str(sys.argv[-1])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
                     
            message2print = "{(Step 2/12b) got" + "; sys-argv-0-got-by-main-of-Popen4Xi: " + str(sys.argv[0])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
      
            message2print = "{(Step 2/12c) got" + "; sys-argv-got-by-main-of-Popen4Xi: " + str(sys.argv)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
            
            # { main : main ; got sys.argv[-1]: test_all_methods4xi.py; time: 21:00:37 06/12/09 FLE Daylight Time}
            # { main : main ; got sys.argv[0]: test_all_methods4xi.py; time: 21:00:37 06/12/09 FLE Daylight Time}
            # { main : main ; got sys.argv: ['test_all_methods4xi.py']; time: 21:00:37 06/12/09 FLE Daylight Time}
            
            
            message2print = "{(Step 2/12d) do" + "; Step-2-of-12-do-by-main-of-Popen4Xi: " + "*** assert is_forking(sys.argv) to be solved; skipped due to assert error ***" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/12)           
            # ----------------------------------------------------------------------------------------- 
            handle2xi = int(sys.argv[-1])
            
            message2print = "{(Step 3/12) got" + "; handle2xi-got-by-main-of-Popen4Xi: " + str(handle2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
                
            
            # -----------------------------------------------------------------------------------------
            # (Step 4/12)           
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 4/12a) passing" + "; handle2xi-passing-to-msvcrt-open-osfhandle-by-main-of-Popen4Xi: " + str(handle2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            
            message2print = "{(Step 4/12b) passing" + "; os-RDONLY-passing-to-msvcrt-open-osfhandle-by-main-of-Popen4Xi: " + str(os.O_RDONLY) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            
            fd2xi = msvcrt.open_osfhandle(handle2xi, os.O_RDONLY)
            
            message2print = "{(Step 4/12c) got" + "; fd2xi-got-by-main-of-Popen4Xi: " + str(fd2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
              
      
            # -----------------------------------------------------------------------------------------
            # (Step 5/12)           
            # -----------------------------------------------------------------------------------------         
            from_parent2xi = os.fdopen(fd2xi, 'rb')
            
            message2print = "{(Step 5/12) got" + "; from-parent2xi-got-by-main-of-Popen4Xi: " + str(from_parent2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
      
      
            
            # -----------------------------------------------------------------------------------------
            # (Step 6/12)           
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 6/12a) passing" + "; true-passing-to-set-inheriting-current-process4xi-by-main-of-Popen4Xi: " + str(True) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
            message2print = "{(Step 6/12b) calling" + "; set-inheriting-current-process4xi-calling-by-main-of-Popen4Xi: " + str(set_inheriting_current_process4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                                            
            
            set_inheriting_current_process4xi(True) #dragonized way of setting the inheritance for the current process True
            
            message2print = "{(Step 6/12c) performed" + "; set-inheriting-current-process4xi-performed-by-main-of-Popen4Xi: " + str(set_inheriting_current_process4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                       
            
            # -----------------------------------------------------------------------------------------
            # (Step 7/12)           
            # -----------------------------------------------------------------------------------------         
            preparation_data2xi = load(from_parent2xi)
            
            message2print = "{(Step 7/12) got" + "; preparation-data2xi-got-by-main-of-Popen4Xi: " + str(preparation_data2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
             
         
            
            # -----------------------------------------------------------------------------------------
            # (Step 8/12)           
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 8/12a) passing" + "; preparation-data2xi-passing-to-prepare4xi-by-main-of-Popen4Xi: " + str(preparation_data2x) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
              
            
            prepare4xi(preparation_data2xi)
    
            
            message2print = "{(Step 8/12b) performed" + "; prepare4xi-performed-by-main-of-Popen4Xi: " + str(prepare4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
          
                    
            # -----------------------------------------------------------------------------------------
            # (Step 9/12)           
            # -----------------------------------------------------------------------------------------         
            self = load(from_parent2xi)
            
            message2print = "{(Step 9/12) got" + "; popen2xi-got-by-main-of-Popen4Xi: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
      
                 
            
            # -----------------------------------------------------------------------------------------
            # (Step 10/12)           
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 10/12a) passing" + "; false-passing-to-set-inheriting-current-process4xi-by-main-of-Popen4Xi: " + str(False) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
              
            
            set_inheriting_current_process4xi(False) #dragonized way of setting the inheritance for the current process False
    
            
            message2print = "{(Step 10/12b) performed" + "; set-inheriting-current-process4xi-performed-by-main-of-Popen4Xi: " + str(set_inheriting_current_process4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                   
            
            # -----------------------------------------------------------------------------------------
            # (Step 11/12)           
            # ----------------------------------------------------------------------------------------- 
            from_parent2xi.close()
    
            
            message2print = "{(Step 11/12a) performed" + "; from-parent2xi-close-performed-by-main-of-Popen4Xi: " + str(from_parent2xi.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
          
            message2print = "{(Step 11/12b) calling" + "; popen2xi-bootstrap-calling-by-main-of-Popen4Xi: " + str(self._bootstrap) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                            
            code2exit = self._bootstrap()
            
            message2print = "{(Step 11/12c) got" + "; code2exit-got-by-main-of-Popen4Xi: " + str(code2exit)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
              
            # -----------------------------------------------------------------------------------------
            # (Step 12/12)          
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(Step 12/12a) passing" + "; code2exit-passing-to-exit-by-main-of-Popen4Xi: " + str(code2exit) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                     
            
            message2print = "{(Step 12/12b) calling" + "; exit-calling-by-main-of-Popen4Xi: " + str(exit) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                            
            exit(code2exit)
    #</function4xi>
 

        
    # ---------------------------------------------------------------------------------------------
    #<function4xi function = "reduce-connection-of-Popen4Xi" usage = "
    #     Make (Pipe)Connection picklable
    #         ForkingPickler.register(connection, reduce_connection) <--- in this module
    #"> 
    #<note4xi note = "Windows OS"/>
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reduce_connection(conn):
      
                       
        # -----------------------------------------------------------------------------------------    
        # (Step 1/4)  
        # -----------------------------------------------------------------------------------------                 

        class2print = "reduce_connection"
        method2print = "reduce_connection"
        
               
        message2print = "{(Step 1/4a) started" + "; reduce-connection-of-Popen4Xi-started: " + str(reduce_connection)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        message2print = "{(Step 1/4b) passed" + "; conn-passed-to-reduce-connection-of-Popen4Xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #         

        # -------------------------------------------------------------------------------------    
        #
        # -------------------------------------------------------------------------------------                 
        if examine4xi(class2print, method2print):
        
            # ---------------------------------------------------------------------------------    
            # StepOriginal
            # ---------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) passing" + "; conn-passing-to-Popen-reduce-connection-by-reduce-connection-of-Popen4Xi: " + str(conn) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                     
            message2print = "{(StepOriginal) returning" + "; Popen-reduce-connection-function-returning-by-reduce-connection-of-Popen4Xi: " + str(Popen.reduce_connection) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                
            
            return Popen.reduce_connection(conn)
        else:        
        
        
            # -----------------------------------------------------------------------------------------
            # (Step 2/3)          
            # -----------------------------------------------------------------------------------------           
            message2print = "{(Step 2/3a) calling" + "; Popen4Xi-thread-is-spawning-calling-by-reduce-connection-of-Popen4Xi: " + str(Popen4Xi.thread_is_spawning) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                            
                            
            
            if not Popen4Xi.thread_is_spawning():
                
                message2print = "{(Step 2/3b) hit" + "; RuntimeError-hit-by-reduce-connection-of-Popen4Xi: " + " *** By default objects can only be shared between processes using inheritance *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                    
                
            
                raise RuntimeError(
                    'By default %s objects can only be shared between processes\n'
                    'using inheritance' % type(conn).__name__
                    )
                    
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)          
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 3/3a) passing" + "; conn-fileno-passing-to-Popen4Xi-duplicate-for-child-by-reduce-connection-of-Popen4Xi: " + str(conn.fileno) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(Step 3/3b) passing" + "; conn-readable-passing-to-Popen4Xi-duplicate-for-child-by-reduce-connection-of-Popen4Xi: " + str(conn.readable) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                              
            
            message2print = "{(Step 3/3c) passing" + "; conn-writable-passing-to-Popen4Xi-duplicate-for-child-by-reduce-connection-of-Popen4Xi: " + str(conn.writable) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                              
            
            message2print = "{(Step 3/3d) returning" + "; Popen4Xi-duplicate-for-child-returning-by-reduce-connection-of-Popen4Xi: " + str(Popen4Xi.duplicate_for_child) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                       
                    
            return type(conn), (Popen4Xi.duplicate_for_child(conn.fileno()),
                                conn.readable, conn.writable)
    

            
    #</function4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of popen4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

 